@@ -34,7 +34,6 @@ execute ``renku help``:
     Options:
       --version                       Print version number.
       --global-config-path            Print global application's config path.
-      --install-completion            Install completion for the current shell.
       --path <path>                   Location of a Renku repository.
                                       [default: (dynamic)]
       --external-storage / -S, --no-external-storage
@@ -59,6 +58,7 @@ Windows:
 
 If in doubt where to look for the configuration file, you can display its path
 by running ``renku --global-config-path``.
+
 """
 import os
 import sys
@@ -66,45 +66,57 @@ import uuid
 from pathlib import Path
 
 import click
-import click_completion
 import yaml
 from click_plugins import with_plugins
-from pkg_resources import iter_entry_points
 
 from renku.cli.clone import clone
 from renku.cli.config import config
 from renku.cli.dataset import dataset
 from renku.cli.doctor import doctor
+from renku.cli.env import env
 from renku.cli.exception_handler import IssueFromTraceback
 from renku.cli.githooks import githooks as githooks_command
 from renku.cli.graph import graph
 from renku.cli.init import init as init_command
 from renku.cli.log import log
-from renku.cli.login import login, logout, token
+from renku.cli.login import credentials, login, logout
 from renku.cli.migrate import check_immutable_template_files, migrate, migrationscheck
 from renku.cli.move import move
+from renku.cli.project import project
 from renku.cli.remove import remove
 from renku.cli.rerun import rerun
+from renku.cli.rollback import rollback
 from renku.cli.run import run
 from renku.cli.save import save
 from renku.cli.service import service
-from renku.cli.show import show
 from renku.cli.status import status
 from renku.cli.storage import storage
 from renku.cli.update import update
 from renku.cli.workflow import workflow
 from renku.core.commands.echo import WARNING
-from renku.core.commands.options import install_completion, option_external_storage_requested
+from renku.core.commands.options import option_external_storage_requested
 from renku.core.commands.version import check_version, print_version
 from renku.core.errors import UsageError
-from renku.core.management.client import LocalClient
-from renku.core.management.config import RENKU_HOME, ConfigManagerMixin
-from renku.core.management.repository import default_path
+from renku.core.utils.git import default_path
 
-#: Monkeypatch Click application.
-click_completion.init()
+try:
+    from importlib.metadata import entry_points
+except ImportError:
+    from importlib_metadata import entry_points
 
-WARNING_UNPROTECTED_COMMANDS = ["clone", "init", "help", "login", "logout", "service"]
+
+def get_entry_points(name: str):
+    """Get entry points from importlib."""
+    all_entry_points = entry_points()
+
+    if hasattr(all_entry_points, "select"):
+        return all_entry_points.select(group=name)
+    else:
+        # Prior to Python 3.10, this returns a dict instead of the selection interface, which is slightly slower
+        return all_entry_points.get(name, [])
+
+
+WARNING_UNPROTECTED_COMMANDS = ["clone", "init", "help", "login", "logout", "service", "credentials", "env"]
 
 
 def _uuid_representer(dumper, data):
@@ -117,6 +129,8 @@ yaml.add_representer(uuid.UUID, _uuid_representer)
 
 def print_global_config_path(ctx, param, value):
     """Print global application's config path."""
+    from renku.core.management.config import ConfigManagerMixin
+
     if not value or ctx.resilient_parsing:
         return
     click.echo(ConfigManagerMixin().global_config_path)
@@ -128,7 +142,7 @@ def is_allowed_command(ctx):
     return ctx.invoked_subcommand in WARNING_UNPROTECTED_COMMANDS or "-h" in sys.argv or "--help" in sys.argv
 
 
-@with_plugins(iter_entry_points("renku.cli_plugins"))
+@with_plugins(get_entry_points("renku.cli_plugins"))
 @click.group(
     cls=IssueFromTraceback, context_settings={"auto_envvar_prefix": "RENKU", "help_option_names": ["-h", "--help"]}
 )
@@ -143,14 +157,6 @@ def is_allowed_command(ctx):
     is_eager=True,
     help=print_global_config_path.__doc__,
 )
-@click.option(
-    "--install-completion",
-    is_flag=True,
-    callback=install_completion,
-    expose_value=False,
-    is_eager=True,
-    help=install_completion.__doc__,
-)
 @click.option(
     "--path", show_default=True, metavar="<path>", default=default_path, help="Location of a Renku repository."
 )
@@ -167,8 +173,17 @@ def is_allowed_command(ctx):
 @click.pass_context
 def cli(ctx, path, external_storage_requested):
     """Check common Renku commands used in various situations."""
+    from renku.core.management import RENKU_HOME
+    from renku.core.management.client import LocalClient
+    from renku.core.management.migrations.utils import OLD_METADATA_PATH
+    from renku.core.management.repository import RepositoryApiMixin
+    from renku.core.metadata.database import Database
+
     renku_path = Path(path) / RENKU_HOME
-    if not renku_path.exists() and not is_allowed_command(ctx):
+    old_metadata = renku_path / OLD_METADATA_PATH
+    new_metadata = renku_path / RepositoryApiMixin.DATABASE_PATH / Database.ROOT_OID
+
+    if not old_metadata.exists() and not new_metadata.exists() and not is_allowed_command(ctx):
         raise UsageError(
             (
                 "`{0}` is not a renku repository.\n"
@@ -195,6 +210,7 @@ cli.add_command(clone)
 cli.add_command(config)
 cli.add_command(dataset)
 cli.add_command(doctor)
+cli.add_command(env)
 cli.add_command(githooks_command)
 cli.add_command(graph)
 cli.add_command(init_command)
@@ -205,14 +221,15 @@ cli.add_command(migrate)
 cli.add_command(migrationscheck)
 cli.add_command(check_immutable_template_files)
 cli.add_command(move)
+cli.add_command(project)
 cli.add_command(remove)
 cli.add_command(rerun)
+cli.add_command(rollback)
 cli.add_command(run)
 cli.add_command(save)
-cli.add_command(show)
 cli.add_command(status)
 cli.add_command(storage)
-cli.add_command(token)
+cli.add_command(credentials)
 cli.add_command(update)
 cli.add_command(workflow)
 cli.add_command(service)
