@@ -20,6 +20,10 @@ r"""Renku CLI commands for handling of datasets.
 Manipulating datasets
 ~~~~~~~~~~~~~~~~~~~~~
 
+.. image:: ../_static/asciicasts/dataset-create.delay.gif
+   :width: 850
+   :alt: Create a Dataset
+
 Creating an empty dataset inside a Renku project:
 
 .. code-block:: console
@@ -27,6 +31,12 @@ Creating an empty dataset inside a Renku project:
     $ renku dataset create my-dataset
     Creating a dataset ... OK
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset create <dataset>
+   :description: Create a new dataset.
+   :extended:
+
 You can pass the following options to this command to set various metadata for
 the dataset.
 
@@ -45,10 +55,17 @@ the dataset.
 | -k, --keyword     | Dataset's keywords. Pass multiple times for a list   |
 |                   | of keywords.                                         |
 +-------------------+------------------------------------------------------+
+| -m, --metadata    | Path to file containing custom JSON-LD metadata to   |
+|                   | be added to the dataset.                             |
++-------------------+------------------------------------------------------+
 
-Editing a dataset's metadata
+Editing a dataset's metadata:
 
-Use ``edit`` subcommand to change metadata of a dataset. You can edit the same
+.. image:: ../_static/asciicasts/dataset-edit.delay.gif
+   :width: 850
+   :alt: Editing a Dataset
+
+Use the ``edit`` sub-command to change metadata of a dataset. You can edit the same
 set of metadata as the create command by passing the options described in the
 table above.
 
@@ -67,6 +84,12 @@ Listing all datasets:
     0ad1cb9a  some-dataset   Some Dataset
     9436e36c  my-dataset     My Dataset
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset ls
+   :description: List all datasets in the project.
+   :extended:
+
 You can select which columns to display by using ``--columns`` to pass a
 comma-separated list of column names:
 
@@ -95,13 +118,17 @@ flag for it:
 
 Showing dataset details:
 
-. code-block:: console
+.. code-block:: console
 
     $ renku dataset show some-dataset
     Name: some-dataset
     Created: 2020-12-09 13:52:06.640778+00:00
     Creator(s): John Doe<john.doe@example.com> [SDSC]
     Keywords: Dataset, Data
+    Annotations:
+    [
+      {...}
+    ]
     Title: Some Dataset
     Description:
     Just some dataset
@@ -114,9 +141,19 @@ Deleting a dataset:
     OK
 
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset rm <dataset>
+   :description: Remove a dataset.
+   :extended:
+
+
 Working with data
 ~~~~~~~~~~~~~~~~~
 
+.. image:: ../_static/asciicasts/dataset-add.delay.gif
+   :width: 850
+   :alt: Add data to a Dataset
 
 Adding data to the dataset:
 
@@ -124,6 +161,13 @@ Adding data to the dataset:
 
     $ renku dataset add my-dataset http://data-url
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset add <dataset> <url>
+   :description: Add data from <url> to a dataset. <url> can be a local
+                 file path, an http(s) address or a Git git+http or git+ssh repository.
+   :extended:
+
 This will copy the contents of ``data-url`` to the dataset and add it
 to the dataset metadata.
 
@@ -188,6 +232,14 @@ will yield:
           new-subdir/
             datafile
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset add <dataset> --source <path>
+             [--destination <rel-path>] <git-url>
+   :description: Add only data in <path> from Git. With --destination:
+                 location the data is copied to.
+   :extended:
+
 To add a specific version of files, use ``--ref`` option for selecting a
 branch, commit, or tag. The value passed to this option must be a valid
 reference in the remote Git repository.
@@ -220,6 +272,12 @@ this limitation, the ``--include`` and ``--exclude`` flags are not compatible
 with those datasets. Modifying those datasets locally will prevent them from
 being updated.
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset update <dataset>
+   :description: Update files in a dataset based on their source.
+   :extended:
+
 The update command also checks for file changes in the project and updates
 datasets' metadata accordingly.
 
@@ -252,6 +310,13 @@ point in time. A tag can be added like this:
 
     $ renku dataset tag my-dataset 1.0 -d "Version 1.0 tag"
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset tag <dataset> <tag> [-d <desc>]
+   :description: Add a tag to the current version of the dataset, with
+                 description <desc>.
+   :extended:
+
 A list of all tags can be seen by running:
 
 .. code-block:: console
@@ -261,6 +326,12 @@ A list of all tags can be seen by running:
     -------------------  ------  ---------------  ----------  ----------------
     2020-09-19 17:29:13  1.0     Version 1.0 tag  my-dataset  6c19a8d31545b...
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset ls-tags <dataset>
+   :description: List all tags for a dataset.
+   :extended:
+
 
 A tag can be removed with:
 
@@ -268,6 +339,12 @@ A tag can be removed with:
 
     $ renku dataset rm-tags my-dataset 1.0
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset rm-tags <dataset> <tags...>
+   :description: Remove tags from a dataset.
+   :extended:
+
 
 Importing data from other Renku projects:
 
@@ -291,6 +368,10 @@ knowing the dataset's ID.
 
 Importing data from an external provider:
 
+.. image:: ../_static/asciicasts/dataset-import.delay.gif
+   :width: 850
+   :alt: Import a Dataset
+
 .. code-block:: console
 
     $ renku dataset import 10.5281/zenodo.3352150
@@ -302,6 +383,13 @@ or ``doi:10.5281/zenodo.3352150``) and full URLs (e.g.
 ``http://zenodo.org/record/3352150``). A tag with the remote version of the
 dataset is automatically created.
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset import <uri>
+   :description: Import a dataset. <uri> can be a Renku, Zenodo or Dataverse
+                 URL or DOI.
+   :extended:
+
 Exporting data to an external provider:
 
 .. code-block:: console
@@ -314,6 +402,13 @@ can chose if the repository `HEAD` version or one of the tags should be
 exported. The remote version will be set to the local tag that is being
 exported.
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset export <dataset> <provider>
+   :description: Export the dataset <dataset> to <provider>. Providers:
+                 Zenodo, Dataverse.
+   :extended:
+
 To export to a Dataverse provider you must pass Dataverse server's URL and
 the name of the parent dataverse where the dataset will be exported to.
 Server's URL is stored in your Renku setting and you don't need to pass it
@@ -340,6 +435,12 @@ Listing all files in the project associated with a dataset.
     my-dataset           2020-02-28 16:49:02  data/my-dataset/weather/file2
     my-dataset           2020-02-28 16:49:02  data/my-dataset/weather/file3  *
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset ls-files
+   :description: List all dataset files in project.
+   :extended:
+
 You can select which columns to display by using ``--columns`` to pass a
 comma-separated list of column names:
 
@@ -376,6 +477,12 @@ Unlink a file from a dataset:
     $ renku dataset unlink my-dataset --include file1
     OK
 
+.. cheatsheet::
+   :group: Datasets
+   :command: $ renku dataset unlink <dataset> [--include <path|pattern>]
+   :description: Remove files from a dataset.
+   :extended:
+
 Unlink all files within a directory from a dataset:
 
 .. code-block:: console
@@ -398,33 +505,25 @@ Unlink all files from a dataset:
     only the dataset record.
 """
 
+import json
+from pathlib import Path
+
 import click
-import requests
-from rich.console import Console
-from rich.markdown import Markdown
 
 from renku.cli.utils.callback import ClickCallback
-from renku.core import errors
-from renku.core.commands.dataset import (
-    add_to_dataset,
-    create_dataset,
-    edit_dataset,
-    export_dataset,
-    file_unlink,
-    import_dataset,
-    list_datasets,
-    list_files,
-    list_tags,
-    remove_dataset,
-    remove_dataset_tags,
-    show_dataset,
-    tag_dataset,
-    update_datasets,
-)
 from renku.core.commands.format.dataset_files import DATASET_FILES_COLUMNS, DATASET_FILES_FORMATS
 from renku.core.commands.format.dataset_tags import DATASET_TAGS_FORMATS
 from renku.core.commands.format.datasets import DATASETS_COLUMNS, DATASETS_FORMATS
-from renku.core.commands.providers import ProviderFactory
+
+
+def _complete_datasets(ctx, param, incomplete):
+    from renku.core.commands.dataset import search_datasets
+
+    try:
+        result = search_datasets().build().execute(name=incomplete)
+        return result.output
+    except Exception:
+        return []
 
 
 @click.group()
@@ -433,7 +532,6 @@ def dataset():
 
 
 @dataset.command("ls")
-@click.option("--revision", default=None)
 @click.option("--format", type=click.Choice(DATASETS_FORMATS), default="tabular", help="Choose an output format.")
 @click.option(
     "-c",
@@ -444,9 +542,11 @@ def dataset():
     help="Comma-separated list of column to display: {}.".format(", ".join(DATASETS_COLUMNS.keys())),
     show_default=True,
 )
-def list_dataset(revision, format, columns):
+def list_dataset(format, columns):
     """List datasets."""
-    result = list_datasets().lock_dataset().build().execute(revision=revision, format=format, columns=columns)
+    from renku.core.commands.dataset import list_datasets
+
+    result = list_datasets().lock_dataset().build().execute(format=format, columns=columns)
     click.echo(result.output)
 
 
@@ -462,17 +562,38 @@ def list_dataset(revision, format, columns):
     multiple=True,
     help="Creator's name, email, and affiliation. Accepted format is 'Forename Surname <email> [affiliation]'.",
 )
-@click.option("-k", "--keyword", default=None, multiple=True, type=click.STRING, help="List of keywords or tags.")
-def create(name, title, description, creators, keyword):
+@click.option(
+    "-m",
+    "--metadata",
+    default=None,
+    type=click.Path(exists=True, dir_okay=False),
+    help="Custom metadata to be associated with the dataset.",
+)
+@click.option("-k", "--keyword", default=None, multiple=True, type=click.STRING, help="List of keywords.")
+def create(name, title, description, creators, metadata, keyword):
     """Create an empty dataset in the current repo."""
+    from renku.core.commands.dataset import create_dataset
+
     communicator = ClickCallback()
     creators = creators or ()
 
+    custom_metadata = None
+
+    if metadata:
+        custom_metadata = json.loads(Path(metadata).read_text())
+
     result = (
         create_dataset()
         .with_communicator(communicator)
         .build()
-        .execute(name=name, title=title, description=description, creators=creators, keywords=keyword)
+        .execute(
+            name=name,
+            title=title,
+            description=description,
+            creators=creators,
+            keywords=keyword,
+            custom_metadata=custom_metadata,
+        )
     )
 
     new_dataset = result.output
@@ -482,7 +603,7 @@ def create(name, title, description, creators, keyword):
 
 
 @dataset.command()
-@click.argument("name")
+@click.argument("name", shell_complete=_complete_datasets)
 @click.option("-t", "--title", default=None, type=click.STRING, help="Title of the dataset.")
 @click.option("-d", "--description", default=None, type=click.STRING, help="Dataset's description.")
 @click.option(
@@ -493,12 +614,26 @@ def create(name, title, description, creators, keyword):
     multiple=True,
     help="Creator's name, email, and affiliation. " "Accepted format is 'Forename Surname <email> [affiliation]'.",
 )
+@click.option(
+    "-m",
+    "--metadata",
+    default=None,
+    type=click.Path(exists=True, dir_okay=False),
+    help="Custom metadata to be associated with the dataset.",
+)
 @click.option("-k", "--keyword", default=None, multiple=True, type=click.STRING, help="List of keywords or tags.")
-def edit(name, title, description, creators, keyword):
+def edit(name, title, description, creators, metadata, keyword):
     """Edit dataset metadata."""
+    from renku.core.commands.dataset import edit_dataset
+
     creators = creators or ()
     keywords = keyword or ()
 
+    custom_metadata = None
+
+    if metadata:
+        custom_metadata = json.loads(Path(metadata).read_text())
+
     result = (
         edit_dataset()
         .build()
@@ -509,6 +644,7 @@ def edit(name, title, description, creators, keyword):
             creators=creators,
             keywords=keywords,
             skip_image_update=True,
+            custom_metadata=custom_metadata,
         )
     )
 
@@ -529,9 +665,12 @@ def edit(name, title, description, creators, keyword):
 
 
 @dataset.command("show")
-@click.argument("name")
+@click.argument("name", shell_complete=_complete_datasets)
 def show(name):
     """Show metadata of a dataset."""
+    from renku.core.commands.dataset import show_dataset
+    from renku.core.utils.os import print_markdown
+
     result = show_dataset().build().execute(name=name)
     ds = result.output
 
@@ -552,15 +691,19 @@ def show(name):
     if ds["version"]:
         click.echo(click.style("Version: ", bold=True, fg="magenta") + ds.get("version", ""))
 
+    if ds["annotations"]:
+        click.echo(click.style("Annotations: ", bold=True, fg="magenta"))
+        click.echo(json.dumps(ds.get("annotations", ""), indent=2))
+
     click.echo(click.style("Title: ", bold=True, fg="magenta") + click.style(ds.get("title", ""), bold=True))
 
     click.echo(click.style("Description: ", bold=True, fg="magenta"))
-    Console().print(Markdown(ds.get("description", "") or ""))
+    print_markdown(ds.get("description", "") or "")
 
 
 @dataset.command()
-@click.argument("name")
-@click.argument("urls", nargs=-1)
+@click.argument("name", shell_complete=_complete_datasets)
+@click.argument("urls", type=click.Path(), nargs=-1)
 @click.option("-e", "--external", is_flag=True, help="Creates a link to external data.")
 @click.option("--force", is_flag=True, help="Allow adding otherwise ignored files.")
 @click.option("-o", "--overwrite", is_flag=True, help="Overwrite existing files.")
@@ -574,6 +717,8 @@ def show(name):
 @click.option("--ref", default=None, help="Add files from a specific commit/tag/branch.")
 def add(name, urls, external, force, overwrite, create, sources, destination, ref):
     """Add data to a dataset."""
+    from renku.core.commands.dataset import add_to_dataset
+
     communicator = ClickCallback()
     add_to_dataset().with_communicator(communicator).build().execute(
         urls=urls,
@@ -590,7 +735,7 @@ def add(name, urls, external, force, overwrite, create, sources, destination, re
 
 
 @dataset.command("ls-files")
-@click.argument("names", nargs=-1)
+@click.argument("names", nargs=-1, shell_complete=_complete_datasets)
 @click.option(
     "--creators",
     help="Filter files which where authored by specific creators. Multiple creators are specified by comma.",
@@ -609,17 +754,26 @@ def add(name, urls, external, force, overwrite, create, sources, destination, re
 )
 def ls_files(names, creators, include, exclude, format, columns):
     """List files in dataset."""
-    result = list_files().lock_dataset().build().execute(names, creators, include, exclude, format, columns)
+    from renku.core.commands.dataset import list_files
+
+    result = (
+        list_files()
+        .lock_dataset()
+        .build()
+        .execute(datasets=names, creators=creators, include=include, exclude=exclude, format=format, columns=columns)
+    )
     click.echo(result.output)
 
 
 @dataset.command()
-@click.argument("name")
+@click.argument("name", shell_complete=_complete_datasets)
 @click.option("-I", "--include", multiple=True, help="Include files matching given pattern.")
 @click.option("-X", "--exclude", multiple=True, help="Exclude files matching given pattern.")
 @click.option("-y", "--yes", is_flag=True, help="Confirm unlinking of all files.")
 def unlink(name, include, exclude, yes):
     """Remove matching files from a dataset."""
+    from renku.core.commands.dataset import file_unlink
+
     communicator = ClickCallback()
     file_unlink().with_communicator(communicator).build().execute(name=name, include=include, exclude=exclude, yes=yes)
     click.secho("OK", fg="green")
@@ -629,36 +783,44 @@ def unlink(name, include, exclude, yes):
 @click.argument("name")
 def remove(name):
     """Delete a dataset."""
+    from renku.core.commands.dataset import remove_dataset
+
     remove_dataset().build().execute(name)
     click.secho("OK", fg="green")
 
 
 @dataset.command("tag")
-@click.argument("name")
+@click.argument("name", shell_complete=_complete_datasets)
 @click.argument("tag")
 @click.option("-d", "--description", default="", help="A description for this tag")
 @click.option("--force", is_flag=True, help="Allow overwriting existing tags.")
 def tag(name, tag, description, force):
     """Create a tag for a dataset."""
-    tag_dataset().build().execute(name, tag, description, force)
+    from renku.core.commands.dataset import add_dataset_tag_command
+
+    add_dataset_tag_command().build().execute(name=name, tag=tag, description=description, force=force)
     click.secho("OK", fg="green")
 
 
 @dataset.command("rm-tags")
-@click.argument("name")
+@click.argument("name", shell_complete=_complete_datasets)
 @click.argument("tags", nargs=-1)
 def remove_tags(name, tags):
     """Remove tags from a dataset."""
-    remove_dataset_tags().build().execute(name, tags)
+    from renku.core.commands.dataset import remove_dataset_tags_command
+
+    remove_dataset_tags_command().build().execute(name=name, tags=tags)
     click.secho("OK", fg="green")
 
 
 @dataset.command("ls-tags")
-@click.argument("name")
+@click.argument("name", shell_complete=_complete_datasets)
 @click.option("--format", type=click.Choice(DATASET_TAGS_FORMATS), default="tabular", help="Choose an output format.")
 def ls_tags(name, format):
     """List all tags of a dataset."""
-    result = list_tags().lock_dataset().build().execute(name, format)
+    from renku.core.commands.dataset import list_tags_command
+
+    result = list_tags_command().lock_dataset().build().execute(name=name, format=format)
     click.echo(result.output)
 
 
@@ -668,7 +830,9 @@ def export_provider_argument(*param_decls, **attrs):
     def wrapper(f):
         from click import argument
 
-        providers = [k.lower() for k, p in ProviderFactory.PROVIDERS.items() if p.supports_export]
+        from renku.core.commands.providers import ProviderFactory
+
+        providers = [k.lower() for k, p in ProviderFactory.providers().items() if p.supports_export]
         f = argument("provider", type=click.Choice(providers))(f)
         return f
 
@@ -681,8 +845,10 @@ def export_provider_options(*param_decls, **attrs):
     def wrapper(f):
         from click_option_group import optgroup
 
+        from renku.core.commands.providers import ProviderFactory
+
         providers = [
-            (k, v) for k, v in ProviderFactory.PROVIDERS.items() if v.supports_export and v.export_parameters()
+            (k, v) for k, v in ProviderFactory.providers().items() if v.supports_export and v.export_parameters()
         ]
 
         for i, (name, provider) in enumerate(providers):
@@ -705,19 +871,22 @@ def export_provider_options(*param_decls, **attrs):
 
 
 @dataset.command("export")
-@click.argument("name")
+@click.argument("name", shell_complete=_complete_datasets)
 @export_provider_argument()
 @click.option("-p", "--publish", is_flag=True, help="Automatically publish exported dataset.")
 @click.option("-t", "--tag", help="Dataset tag to export")
 @export_provider_options()
 def export_(name, provider, publish, tag, **kwargs):
     """Export data to 3rd party provider."""
+    from renku.core import errors
+    from renku.core.commands.dataset import export_dataset
+
     try:
         communicator = ClickCallback()
         export_dataset().lock_dataset().with_communicator(communicator).build().execute(
             name=name, provider_name=provider, publish=publish, tag=tag, **kwargs
         )
-    except (ValueError, errors.InvalidAccessToken, errors.DatasetNotFound, requests.HTTPError) as e:
+    except (ValueError, errors.InvalidAccessToken, errors.DatasetNotFound, errors.RequestError) as e:
         raise click.BadParameter(e)
 
     click.secho("OK", fg="green")
@@ -733,6 +902,8 @@ def import_(uri, name, extract, yes):
 
     Supported providers: [Dataverse, Renku, Zenodo]
     """
+    from renku.core.commands.dataset import import_dataset
+
     communicator = ClickCallback()
     import_dataset().with_communicator(communicator).build().execute(uri=uri, name=name, extract=extract, yes=yes)
 
@@ -741,7 +912,7 @@ def import_(uri, name, extract, yes):
 
 
 @dataset.command("update")
-@click.argument("names", nargs=-1)
+@click.argument("names", nargs=-1, shell_complete=_complete_datasets)
 @click.option(
     "--creators",
     help="Filter files which where authored by specific creators. Multiple creators are specified by comma.",
@@ -753,6 +924,8 @@ def import_(uri, name, extract, yes):
 @click.option("-e", "--external", is_flag=True, help="Update external data.")
 def update(names, creators, include, exclude, ref, delete, external):
     """Updates files in dataset from a remote Git repo."""
+    from renku.core.commands.dataset import update_datasets
+
     communicator = ClickCallback()
     update_datasets().with_communicator(communicator).build().execute(
         names=list(names),
