@@ -15,18 +15,25 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""Check your system and repository for potential problems."""
+"""Check your system and repository for potential problems.
+
+.. cheatsheet::
+   :group: Misc
+   :command: $ renku doctor
+   :description: Check your system and repository for potential problems.
+   :extended:
+"""
 import textwrap
 
 import click
 
-from renku.core.commands.doctor import DOCTOR_INFO, doctor_check_command
-
 
 @click.command()
 @click.pass_context
 def doctor(ctx):
     """Check your system and repository for potential problems."""
+    from renku.core.commands.doctor import DOCTOR_INFO, doctor_check_command
+
     click.secho("\n".join(textwrap.wrap(DOCTOR_INFO)) + "\n", bold=True)
     is_ok, problems = doctor_check_command().build().execute().output
 
