@@ -37,6 +37,11 @@ or:
 This creates a new subdirectory named ``.renku`` that contains all the
 necessary files for managing the project configuration.
 
+Every project requires a ``name`` that can either be provided using
+``--name`` or automatically taken from the target folder.
+
+You can also provide a description for a project using ``--description``.
+
 If provided directory does not exist, it will be created.
 
 Use a different template
@@ -102,10 +107,29 @@ If you don't provide the required parameters through the option
 ``-parameter``, you will be asked to provide them. Empty values are allowed
 and passed to the template initialization function.
 
-.. note:: Every project requires a ``name`` that can either be provided using
-   ``--name`` or automatically taken from the target folder. This is
-   also considered as a special parameter, therefore it's automatically added
-   to the list of parameters forwarded to the ``init`` command.
+.. note::
+
+    Project's ``name`` is considered as a special parameter and it's
+    automatically added to the list of parameters forwarded to the ``init``
+    command.
+
+Provide custom metadata
+~~~~~~~~~~~~~~~~~~~~~~~
+
+Custom metadata can be added to the projects knowledge graph by writing
+it to a json file and passing that via the `--metadata` option.
+
+.. code-block:: console
+
+    $ echo '{"@id": "https://example.com/id1", \
+        "@type": "https://schema.org/Organization", \
+        "https://schema.org/legalName": "ETHZ"}' > metadata.json
+
+    $ renku init --template-id python-minimal --parameter \
+    "description"="my new shiny project" --metadata metadata.json
+
+    Initializing new Renku repository... OK
+
 
 Update an existing project
 ~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -164,17 +188,13 @@ was not installed previously.
 
 """
 
-import configparser
+import json
 import os
 from pathlib import Path
-from tempfile import mkdtemp
 
 import click
-from git import Repo
 
-from renku.cli.utils.callback import ClickCallback
 from renku.core import errors
-from renku.core.commands.init import init_command
 from renku.core.commands.options import option_external_storage_requested
 
 _GITLAB_CI = ".gitlab-ci.yml"
@@ -224,22 +244,11 @@ def resolve_data_directory(data_dir, path):
     return data_dir
 
 
-def check_git_user_config():
-    """Check that git user information is configured."""
-    dummy_git_folder = mkdtemp()
-    repo = Repo.init(dummy_git_folder)
-    git_config = repo.config_reader()
-    try:
-        git_config.get_value("user", "name", None)
-        git_config.get_value("user", "email", None)
-        return True
-    except (configparser.NoOptionError, configparser.NoSectionError):
-        return False
-
-
 @click.command()
 @click.argument("path", default=".", type=click.Path(writable=True, file_okay=False, resolve_path=True))
 @click.option("-n", "--name", callback=validate_name, help="Provide a custom project name.")
+@click.option("--description", help="Provide a description for the project.")
+@click.option("-k", "--keyword", default=None, multiple=True, type=click.STRING, help="List of keywords.")
 @click.option(
     "--data-dir",
     default=None,
@@ -255,7 +264,7 @@ def check_git_user_config():
 @click.option(
     "-p",
     "--parameter",
-    "metadata",
+    "parameters",
     multiple=True,
     type=click.STRING,
     callback=parse_parameters,
@@ -264,6 +273,14 @@ def check_git_user_config():
         'Please specify the values as follow: --parameter "param1"="value"'
     ),
 )
+@click.option(
+    "-m",
+    "--metadata",
+    "metadata",
+    default=None,
+    type=click.Path(exists=True, dir_okay=False),
+    help="Custom metadata to be associated with the project.",
+)
 @click.option("-l", "--list-templates", is_flag=True, help="List templates available in the template-source.")
 @click.option("-d", "--describe", is_flag=True, help="Show description for templates and parameters")
 @click.option("--force", is_flag=True, help="Override target path.")
@@ -275,10 +292,13 @@ def init(
     external_storage_requested,
     path,
     name,
+    description,
+    keyword,
     template_id,
     template_index,
     template_source,
     template_ref,
+    parameters,
     metadata,
     list_templates,
     force,
@@ -287,31 +307,35 @@ def init(
     initial_branch,
 ):
     """Initialize a project in PATH. Default is the current path."""
+    from renku.cli.utils.callback import ClickCallback
+    from renku.core.commands.init import init_command
+    from renku.core.utils.git import check_global_git_user_is_configured
+
     data_dir = resolve_data_directory(data_dir, path)
 
-    if not check_git_user_config():
-        raise errors.ConfigurationError(
-            "The user name and email are not configured. "
-            'Please use the "git config" command to configure them.\n\n'
-            '\tgit config --global --add user.name "John Doe"\n'
-            "\tgit config --global --add user.email "
-            '"john.doe@example.com"\n'
-        )
+    check_global_git_user_is_configured()
 
     if template_ref and not template_source:
         raise errors.ParameterError("Can't use '--template-ref' without specifying '--template-source'")
 
+    custom_metadata = None
+    if metadata:
+        custom_metadata = json.loads(Path(metadata).read_text())
+
     communicator = ClickCallback()
     init_command().with_communicator(communicator).build().execute(
         ctx=ctx,
         external_storage_requested=external_storage_requested,
         path=path,
         name=name,
+        description=description,
+        keywords=keyword,
         template_id=template_id,
         template_index=template_index,
         template_source=template_source,
         template_ref=template_ref,
-        metadata=metadata,
+        metadata=parameters,
+        custom_metadata=custom_metadata,
         list_templates=list_templates,
         force=force,
         describe=describe,
