@@ -65,7 +65,6 @@ endpoints are removed.
 import click
 
 from renku.cli.utils.callback import ClickCallback
-from renku.core.commands.login import login_command, logout_command, token_command
 
 
 @click.command()
@@ -74,6 +73,8 @@ from renku.core.commands.login import login_command, logout_command, token_comma
 @click.option("--yes", is_flag=True, default=False, hidden=True, help="Do not ask for user confirmation.")  # For tests
 def login(endpoint, git, yes):
     """Log in to the platform."""
+    from renku.core.commands.login import login_command
+
     communicator = ClickCallback()
     login_command().with_communicator(communicator).build().execute(endpoint=endpoint, git_login=git, yes=yes)
     click.secho("Successfully logged in.", fg="green")
@@ -83,6 +84,8 @@ def login(endpoint, git, yes):
 @click.argument("endpoint", required=False, default=None)
 def logout(endpoint):
     """Logout from the platform and delete credentials."""
+    from renku.core.commands.login import logout_command
+
     communicator = ClickCallback()
     logout_command().with_communicator(communicator).build().execute(endpoint=endpoint)
     click.secho("Successfully logged out.", fg="green")
@@ -91,7 +94,9 @@ def logout(endpoint):
 @click.command(hidden=True)
 @click.option("--hostname", default=None, hidden=True, help="Remote hostname.")
 @click.argument("command")
-def token(command, hostname):
-    """A git credential helper for returning renku token."""
+def credentials(command, hostname):
+    """A git credential helper for returning renku user/token."""
+    from renku.core.commands.login import credentials_command
+
     communicator = ClickCallback()
-    token_command().with_communicator(communicator).build().execute(command=command, hostname=hostname)
+    credentials_command().with_communicator(communicator).build().execute(command=command, hostname=hostname)
