@@ -38,6 +38,12 @@ by running
 
     $ renku migrate -c
 
+.. cheatsheet::
+   :group: Misc
+   :command: $ renku migrate
+   :description: Migrate old metadata to the current Renku version.
+   :extended:
+
 """
 import json
 import os
@@ -46,31 +52,35 @@ import click
 
 from renku.cli.utils.callback import ClickCallback
 from renku.core.commands.echo import ERROR, INFO
-from renku.core.commands.migrate import (
-    AUTOMATED_TEMPLATE_UPDATE_SUPPORTED,
-    DOCKERFILE_UPDATE_POSSIBLE,
-    MIGRATION_REQUIRED,
-    NON_RENKU_REPOSITORY,
-    TEMPLATE_UPDATE_POSSIBLE,
-    UNSUPPORTED_PROJECT,
-    check_immutable_template_files_command,
-    check_project,
-    migrate_project,
-    migrations_check,
-    migrations_versions,
-)
 from renku.core.errors import MigrationRequired, ProjectNotSupported
 
 
 @click.command()
 @click.option("-c", "--check", is_flag=True, help="Check if migration is required and quit.")
-@click.option("--no-commit", is_flag=True, hidden=True, help="Do not commit changes after the migration.")
 @click.option("-t", "--skip-template-update", is_flag=True, hidden=True, help="Do not update project template files.")
 @click.option(
     "-d", "--skip-docker-update", is_flag=True, hidden=True, help="Do not update Dockerfile to current renku version."
 )
-def migrate(check, no_commit, skip_template_update, skip_docker_update):
+@click.option("-s", "--strict", is_flag=True, hidden=True, help="Abort migrations if an error is raised.")
+@click.option(
+    "--preserve-identifiers",
+    is_flag=True,
+    hidden=True,
+    help="Keep original dataset identifier when KG migrates a project.",
+)
+def migrate(check, skip_template_update, skip_docker_update, strict, preserve_identifiers):
     """Check for migration and migrate to the latest Renku project version."""
+    from renku.core.commands.migrate import (
+        AUTOMATED_TEMPLATE_UPDATE_SUPPORTED,
+        DOCKERFILE_UPDATE_POSSIBLE,
+        MIGRATION_REQUIRED,
+        NON_RENKU_REPOSITORY,
+        TEMPLATE_UPDATE_POSSIBLE,
+        UNSUPPORTED_PROJECT,
+        check_project,
+        migrate_project,
+    )
+
     status = check_project().build().execute().output
 
     template_update_possible = status & TEMPLATE_UPDATE_POSSIBLE and status & AUTOMATED_TEMPLATE_UPDATE_SUPPORTED
@@ -107,10 +117,13 @@ def migrate(check, no_commit, skip_template_update, skip_docker_update):
 
     communicator = ClickCallback()
 
-    command = migrate_project().with_communicator(communicator)
-    if not no_commit:
-        command = command.with_commit()
-    result = command.build().execute(skip_template_update=skip_template_update, skip_docker_update=skip_docker_update)
+    command = migrate_project().with_communicator(communicator).with_commit()
+    result = command.build().execute(
+        skip_template_update=skip_template_update,
+        skip_docker_update=skip_docker_update,
+        strict=strict,
+        preserve_identifiers=preserve_identifiers,
+    )
 
     result, _, _ = result.output
 
@@ -123,46 +136,18 @@ def migrate(check, no_commit, skip_template_update, skip_docker_update):
 @click.command(hidden=True)
 def migrationscheck():
     """Check status of the project and current renku-python version."""
-    latest_version, project_version = migrations_versions().build().execute().output
-    (
-        migration_required,
-        project_supported,
-        template_update_possible,
-        current_template_version,
-        latest_template_version,
-        template_source,
-        template_ref,
-        template_id,
-        automated_update,
-        docker_update_possible,
-    ) = (
-        migrations_check().lock_project().build().execute().output
-    )
+    from renku.core.commands.migrate import migrations_check
 
-    click.echo(
-        json.dumps(
-            {
-                "latest_version": latest_version,
-                "project_version": project_version,
-                "migration_required": migration_required,
-                "project_supported": project_supported,
-                "template_update_possible": template_update_possible,
-                "current_template_version": str(current_template_version),
-                "latest_template_version": str(latest_template_version),
-                "template_source": template_source,
-                "template_ref": template_ref,
-                "template_id": template_id,
-                "automated_update": automated_update,
-                "docker_update_possible": docker_update_possible,
-            }
-        )
-    )
+    result = migrations_check().lock_project().build().execute().output
+    click.echo(json.dumps(result))
 
 
 @click.command(hidden=True)
 @click.argument("paths", type=click.Path(exists=True, dir_okay=True), nargs=-1, required=True)
 def check_immutable_template_files(paths):
     """Check specified paths if they are marked immutable in the template."""
+    from renku.core.commands.migrate import check_immutable_template_files_command
+
     result = check_immutable_template_files_command().build().execute(paths=paths)
     paths = result.output
     if paths:
