@@ -32,6 +32,13 @@ If you have local modification to files, you can save them using
 .. warning:: The username and password for renku save are your gitlab
    user/password, not your renkulab login!
 
+.. cheatsheet::
+   :group: Misc
+   :command: $ renku save [-m <message>]
+   :description: Save (commit) and push all local changes. with optional
+                 message.
+   :extended:
+
 You can additionally supply a message that describes the changes that you
 made by using the ``-m`` or ``--message`` parameter followed by your
 message.
@@ -72,7 +79,6 @@ You can also specify which paths to save:
 import click
 
 from renku.cli.utils.callback import ClickCallback
-from renku.core.commands.save import save_and_push_command
 
 
 @click.command(name="save")
@@ -88,6 +94,8 @@ from renku.core.commands.save import save_and_push_command
 @click.argument("paths", type=click.Path(exists=False, dir_okay=True), nargs=-1)
 def save(message, destination, paths):
     """Save and push local changes."""
+    from renku.core.commands.save import save_and_push_command
+
     communicator = ClickCallback()
     saved_paths, branch = (
         save_and_push_command()
