@@ -21,7 +21,6 @@ import functools
 import os
 
 import click
-from git.remote import RemoteProgress
 
 INFO = click.style("Info: ", bold=True, fg="blue")
 WARNING = click.style("Warning: ", bold=True, fg="yellow")
@@ -33,8 +32,8 @@ def echo_via_pager(*args, **kwargs):
 
     NOTE: The feature is available only on ``less``-based pager.
     """
+    restore = "LESS" not in os.environ
     try:
-        restore = "LESS" not in os.environ
         os.environ.setdefault("LESS", "-iXFR")
         click.echo_via_pager(*args, **kwargs)
     finally:
@@ -45,23 +44,3 @@ def echo_via_pager(*args, **kwargs):
 progressbar = functools.partial(
     click.progressbar, fill_char=click.style(" ", bg="green"), show_pos=True, item_show_func=lambda x: x
 )
-
-
-class GitProgress(RemoteProgress):
-    """Progress printing for GitPython."""
-
-    def __init__(self):
-        """Initialize a Git progress printer."""
-        super().__init__()
-        self._previous_line_length = 0
-
-    def update(self, op_code, cur_count, max_count=None, message=""):
-        """Callback for printing Git operation status."""
-        self._clear_line()
-        print(self._cur_line, end="\r")
-        self._previous_line_length = len(self._cur_line)
-        if (op_code & RemoteProgress.END) != 0:
-            print()
-
-    def _clear_line(self):
-        print(self._previous_line_length * " ", end="\r")
