@@ -16,31 +16,31 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Serializers for dataset list files."""
+
 import re
 from subprocess import PIPE, SubprocessError, run
 
-from humanize import naturalsize
-
-from renku.core.models.datasets import DatasetFileDetailsJson
-
-from .tabulate import tabulate
+from renku.core.commands.format.tabulate import tabulate
+from renku.core.commands.schema.dataset import DatasetFileSchema
+from renku.core.management.command_builder import inject
+from renku.core.management.interface.client_dispatcher import IClientDispatcher
 
 
-def tabular(client, records, *, columns=None):
+def tabular(records, *, columns=None):
     """Format dataset files with a tabular output.
 
-    :param client: LocalClient instance.
     :param records: Filtered collection.
     :param columns: List of columns to display
     """
+
     if not columns:
         columns = "added,creators,dataset,full_path"
 
     if "size" in columns.split(","):
-        _get_lfs_file_sizes(client, records)
+        _get_lfs_file_sizes(records)
 
     if "lfs" in columns.split(","):
-        _get_lfs_tracking(client, records)
+        _get_lfs_tracking(records)
 
     for record in records:
         record.creators = record.dataset.creators
@@ -53,20 +53,28 @@ def tabular(client, records, *, columns=None):
     )
 
 
-def _get_lfs_tracking(client, records):
+@inject.autoparams()
+def _get_lfs_tracking(records, client_dispatcher: IClientDispatcher):
     """Check if files are tracked in git lfs."""
-    paths = [r.path for r in records]
-    attrs = client.find_attr(*paths)
+    client = client_dispatcher.current_client
+
+    paths = (r.path for r in records)
+    attrs = client.repository.get_attributes(*paths)
 
     for record in records:
-        if attrs.get(record.path, {}).get("filter") == "lfs":
+        if attrs.get(str(record.path), {}).get("filter") == "lfs":
             record.is_lfs = True
         else:
             record.is_lfs = False
 
 
-def _get_lfs_file_sizes(client, records):
+@inject.autoparams()
+def _get_lfs_file_sizes(records, client_dispatcher: IClientDispatcher):
     """Try to get file size from Git LFS."""
+    from humanize import naturalsize  # Slow import
+
+    client = client_dispatcher.current_client
+
     lfs_files_sizes = {}
 
     try:
@@ -90,7 +98,9 @@ def _get_lfs_file_sizes(client, records):
                 size = size.replace(" B", "  B")
             lfs_files_sizes[path] = size
 
-    non_lfs_files_sizes = {o.path: o.size for o in client.repo.tree().traverse() if o.path not in lfs_files_sizes}
+    non_lfs_files_sizes = {
+        o.path: o.size for o in client.repository.head.commit.traverse() if o.path not in lfs_files_sizes
+    }
     non_lfs_files_sizes = {k: naturalsize(v).upper().replace("BYTES", " B") for k, v in non_lfs_files_sizes.items()}
 
     for record in records:
@@ -98,28 +108,27 @@ def _get_lfs_file_sizes(client, records):
         record.size = size
 
 
-def jsonld(client, records, **kwargs):
+def jsonld(records, **kwargs):
     """Format dataset files as JSON-LD.
 
-    :param client: LocalClient instance.
     :param records: Filtered collection.
     """
     from renku.core.models.json import dumps
 
-    data = [record.as_jsonld() for record in records]
+    data = [DatasetFileSchema(flattened=True).dump(record) for record in records]
     return dumps(data, indent=2)
 
 
-def json(client, records, **kwargs):
+def json(records, **kwargs):
     """Format dataset files as JSON.
 
-    :param client: LocalClient instance.
     :param records: Filtered collection.
     """
+    from renku.core.models.dataset import DatasetFileDetailsJson
     from renku.core.models.json import dumps
 
-    _get_lfs_file_sizes(client, records)
-    _get_lfs_tracking(client, records)
+    _get_lfs_file_sizes(records)
+    _get_lfs_tracking(records)
 
     for record in records:
         record.creators = record.dataset.creators
@@ -136,8 +145,8 @@ DATASET_FILES_FORMATS = {
 """Valid formatting options."""
 
 DATASET_FILES_COLUMNS = {
-    "added": ("added", None),
-    "commit": ("commit_sha", "commit"),
+    "added": ("date_added", "added"),
+    "commit": ("entity.checksum", "commit"),
     "creators": ("creators_csv", "creators"),
     "creators_full": ("creators_full_csv", "creators"),
     "dataset": ("title", "dataset"),
