@@ -18,13 +18,15 @@
 """Tabular format helper functions."""
 
 from collections import OrderedDict
+from operator import attrgetter
 
 from renku.core import errors
-from renku.core.models.tabulate import tabulate as tabulate_
 
 
-def tabulate(collection, columns, columns_mapping, columns_alignments=None):
+def tabulate(collection, columns, columns_mapping, columns_alignments=None, sort=True, reverse=False):
     """Format collection with a tabular output."""
+    from renku.core.models.tabulate import tabulate as tabulate_
+
     if not columns:
         raise errors.ParameterError("Columns cannot be empty.")
 
@@ -33,11 +35,13 @@ def tabulate(collection, columns, columns_mapping, columns_alignments=None):
     headers, alignments = _make_headers(columns, columns_mapping, columns_alignments)
 
     # Sort based on the first requested field
-    attr = list(headers.keys())[0]
-    try:
-        collection = sorted(collection, key=lambda d: getattr(d, attr))
-    except TypeError:
-        pass
+    if sort:
+        try:
+            attr = list(headers.keys())[0]
+            getter = attrgetter(attr)
+            collection = sorted(collection, key=getter, reverse=reverse)
+        except TypeError:
+            pass
 
     alignments = alignments if collection else None  # To avoid a tabulate bug
 
