@@ -19,7 +19,6 @@
 
 import json
 import re
-import tempfile
 from collections import OrderedDict, namedtuple
 from pathlib import Path
 from tempfile import mkdtemp
@@ -27,19 +26,28 @@ from uuid import uuid4
 
 import attr
 import click
-import git
-import pkg_resources
 import yaml
 
 from renku.core import errors
 from renku.core.commands.git import set_git_home
-from renku.core.incubation.command import Command
-from renku.core.management.config import RENKU_HOME
+from renku.core.management import RENKU_HOME
+from renku.core.management.command_builder.command import Command, inject
+from renku.core.management.interface.client_dispatcher import IClientDispatcher
+from renku.core.management.interface.database_dispatcher import IDatabaseDispatcher
+from renku.core.management.interface.database_gateway import IDatabaseGateway
+from renku.core.management.migrations.utils import OLD_METADATA_PATH
 from renku.core.management.repository import INIT_APPEND_FILES, INIT_KEEP_FILES
+from renku.core.metadata.repository import Repository
 from renku.core.models.tabulate import tabulate
 from renku.core.utils import communication
+from renku.core.utils.git import clone_repository
 from renku.version import __version__, is_release
 
+try:
+    import importlib_resources
+except ImportError:
+    import importlib.resources as importlib_resources
+
 TEMPLATE_MANIFEST = "manifest.yaml"
 
 
@@ -58,11 +66,17 @@ def create_template_sentence(templates, describe=False, instructions=False):
         return None
 
     def extract_variables(template_elem):
-        """Extract variables from tempalte manifest."""
+        """Extract variables from template manifest."""
         if describe:
-            return "\n".join(
-                [f"{variable[0]}: {variable[1]}" for variable in template_elem.get("variables", {}).items()]
-            )
+            descriptions = []
+            for name, variable in template_elem.get("variables", {}).items():
+                variable_type = f', type: {variable["type"]}' if "type" in variable else ""
+                enum_values = f', options: {variable["enum"]}' if "enum" in variable else ""
+                default_value = f', default: {variable["default_value"]}' if "default_value" in variable else ""
+                description = variable["description"]
+
+                descriptions.append(f"{name}: {description}{variable_type}{enum_values}{default_value}")
+            return "\n".join(descriptions)
 
         return ",".join(template_elem.get("variables", {}).keys())
 
@@ -90,7 +104,8 @@ def create_template_sentence(templates, describe=False, instructions=False):
 
 def store_directory(value):
     """Store directory as a new Git home."""
-    Path(value).mkdir(parents=True, exist_ok=True)
+    value = Path(value)
+    value.mkdir(parents=True, exist_ok=True)
     set_git_home(value)
     return value
 
@@ -123,7 +138,7 @@ def select_template_from_manifest(
             repeat = True
 
     if template_index is not None:
-        if template_index > 0 and template_index <= len(template_manifest):
+        if 0 < template_index <= len(template_manifest):
             template_data = template_manifest[template_index - 1]
         else:
             communication.echo(f"The template at index {template_index} is not available.")
@@ -133,6 +148,8 @@ def select_template_from_manifest(
     if prompt and (repeat or not (template_id or template_index)):
         templates = [template_elem for template_elem in template_manifest]
         if len(templates) == 1:
+            if describe:
+                communication.echo(create_template_sentence(templates, describe=describe, instructions=False))
             template_data = templates[0]
         else:
             template_index = communication.prompt(
@@ -148,18 +165,107 @@ def select_template_from_manifest(
     return template_data, template_id
 
 
+def validate_template_variable_value(name, template_variable, value):
+    """Validates template values by type."""
+    if "type" not in template_variable:
+        return True, None, value
+
+    variable_type = template_variable["type"]
+    valid = True
+
+    if variable_type == "string":
+        if not isinstance(value, str):
+            valid = False
+    elif variable_type == "number":
+        try:
+            value = int(value)
+            is_int = True
+        except ValueError:
+            is_int = False
+
+        try:
+            value = float(value)
+            is_float = True
+        except ValueError:
+            is_float = False
+
+        if not is_float and not is_int:
+            valid = False
+    elif variable_type == "boolean":
+        truthy = [True, 1, "1", "true", "True"]
+        falsy = [False, 0, "0", "false", "False"]
+        if value not in truthy and value not in falsy:
+            valid = False
+        else:
+            value = True if value in truthy else False
+    elif variable_type == "enum":
+        if "enum" not in template_variable:
+            raise errors.InvalidTemplateError(
+                f'Template contains variable {name} of type enum but does not provide a corresponding "enum" list.'
+            )
+        possible_values = template_variable["enum"]
+        if value not in possible_values:
+            return (
+                False,
+                f"Value '{value}' is not in list of possible values {possible_values} for template parameter {name}.",
+                value,
+            )
+    else:
+        raise errors.InvalidTemplateError(
+            f"Template contains variable {name} of type {variable_type} which is not supported."
+        )
+
+    if not valid:
+        return False, f"Value '{value}' is not of type {variable_type} required by {name}.", value
+
+    return True, None, value
+
+
+def prompt_for_value(name, template_variable):
+    """Prompt the user for a template value."""
+    valid = False
+    while not valid:
+        variable_type = f', type: {template_variable["type"]}' if "type" in template_variable else ""
+        enum_values = f', options: {template_variable["enum"]}' if "enum" in template_variable else ""
+        default_value = template_variable["default_value"] if "default_value" in template_variable else ""
+
+        value = communication.prompt(
+            msg=(
+                f'The template requires a value for "{name}" ({template_variable["description"]}'
+                f"{variable_type}{enum_values})"
+            ),
+            default=default_value,
+            show_default=bool(default_value),
+        )
+
+        valid, msg, value = validate_template_variable_value(name, template_variable, value)
+
+        if msg:
+            communication.info(msg)
+
+    return value
+
+
 def verify_template_variables(template_data, metadata):
-    """Verifies that template variables are correcly set."""
+    """Verifies that template variables are correctly set."""
     template_variables = template_data.get("variables", {})
     template_variables_keys = set(template_variables.keys())
     input_parameters_keys = set(metadata.keys())
+
+    for key in template_variables:
+        if "description" not in template_variables[key]:
+            raise errors.InvalidTemplateError(f"Template parameter {key} does not contain a description.")
+
+    for key in sorted(template_variables_keys.intersection(input_parameters_keys)):
+        valid, msg, metadata[key] = validate_template_variable_value(key, template_variables[key], metadata[key])
+
+        if not valid:
+            communication.info(msg)
+
+            metadata[key] = prompt_for_value(key, template_variables[key])
+
     for key in template_variables_keys - input_parameters_keys:
-        value = communication.prompt(
-            msg=(f'The template requires a value for "{key}" ' f"({template_variables[key]})"),
-            default="",
-            show_default=False,
-        )
-        metadata[key] = value
+        metadata[key] = prompt_for_value(key, template_variables[key])
 
     # ignore internal variables, i.e. __\w__
     internal_keys = re.compile(r"__\w+__$")
@@ -174,8 +280,11 @@ def verify_template_variables(template_data, metadata):
     return metadata
 
 
-def get_existing_template_files(client, template_path, metadata, force=False):
-    """Gets files in the template that already exists in the repo."""
+@inject.autoparams()
+def get_existing_template_files(template_path, metadata, client_dispatcher: IClientDispatcher, force=False):
+    """Gets files in the template that already exists in the repository."""
+    client = client_dispatcher.current_client
+
     template_files = list(client.get_template_files(template_path, metadata))
 
     existing = []
@@ -189,21 +298,23 @@ def get_existing_template_files(client, template_path, metadata, force=False):
     return existing
 
 
-def create_backup_branch(client, path):
-    """Creates a backup branch of the repo."""
+@inject.autoparams()
+def create_backup_branch(path, client_dispatcher: IClientDispatcher):
+    """Creates a backup branch of the repository."""
+    client = client_dispatcher.current_client
+
     branch_name = None
     if not is_path_empty(path):
-
         try:
-            if client.repo.head.is_valid():
-                commit = client.repo.head.commit.commit
+            if client.repository.head.is_valid():
+                commit = client.repository.head.commit
                 hexsha = commit.hexsha[:7]
 
                 branch_name = f"pre_renku_init_{hexsha}"
 
-                for ref in client.repo.references:
+                for ref in client.repository.branches:
                     if branch_name == ref.name:
-                        branch_name = f"pre_renku_init_{hexsha}_{uuid4().hexsha}"
+                        branch_name = f"pre_renku_init_{hexsha}_{uuid4().hex}"
                         break
 
                 with client.worktree(
@@ -214,30 +325,37 @@ def create_backup_branch(client, path):
                     communication.warn("Saving current data in branch {0}".format(branch_name))
         except AttributeError:
             communication.echo("Warning! Overwriting non-empty folder.")
-        except git.exc.GitCommandError as e:
-            raise errors.GitError(e)
+        except errors.GitCommandError:
+            raise
 
     return branch_name
 
 
+@inject.autoparams()
 def _init(
-    client,
     ctx,
     external_storage_requested,
     path,
     name,
+    description,
+    keywords,
     template_id,
     template_index,
     template_source,
     template_ref,
     metadata,
+    custom_metadata,
     list_templates,
     force,
     describe,
     data_dir,
     initial_branch,
+    client_dispatcher: IClientDispatcher,
+    database_dispatcher: IDatabaseDispatcher,
 ):
     """Initialize a renku project."""
+    client = client_dispatcher.current_client
+
     template_manifest, template_folder, template_source, template_version = fetch_template(
         template_source, template_ref
     )
@@ -255,18 +373,22 @@ def _init(
 
     metadata = verify_template_variables(template_data, metadata)
 
-    # set local path and storage
+    # NOTE: set local path and storage
     store_directory(path)
     if not client.external_storage_requested:
         external_storage_requested = False
+
+    # NOTE: create new copy of LocalClient with modified values
     ctx.obj = client = attr.evolve(
         client, path=path, data_dir=data_dir, external_storage_requested=external_storage_requested
     )
+    client_dispatcher.push_created_client_to_stack(client)
+    database_dispatcher.push_database_to_stack(client.database_path, commit=True)
 
     communication.echo("Initializing Git repository...")
     client.init_repository(force, None, initial_branch=initial_branch)
 
-    # supply additional metadata
+    # NOTE: supply additional metadata
     metadata["__template_source__"] = template_source
     metadata["__template_ref__"] = template_ref
     metadata["__template_id__"] = template_id
@@ -274,13 +396,15 @@ def _init(
     metadata["__sanitized_project_name__"] = ""
     metadata["__repository__"] = ""
     metadata["__project_slug__"] = ""
+    metadata["__project_description__"] = description
     if is_release() and "__renku_version__" not in metadata:
         metadata["__renku_version__"] = __version__
-    metadata["name"] = name
+    metadata["name"] = name  # NOTE: kept for backwards compatibility
+    metadata["__name__"] = name
 
     template_path = template_folder / template_data["folder"]
 
-    existing = get_existing_template_files(client, template_path, metadata, force)
+    existing = get_existing_template_files(template_path=template_path, metadata=metadata, force=force)
 
     append = list(filter(lambda x: x.lower() in INIT_APPEND_FILES, existing))
     existing = list(filter(lambda x: x.lower() not in INIT_APPEND_FILES + INIT_KEEP_FILES, existing))
@@ -300,9 +424,21 @@ def _init(
 
         communication.confirm(f"{message}Proceed?", abort=True, warning=True)
 
-    branch_name = create_backup_branch(client, path)
+    branch_name = create_backup_branch(path=path)
 
-    # clone the repo
+    # Initialize an empty database
+    database_gateway = inject.instance(IDatabaseGateway)
+    database_gateway.initialize()
+
+    # add metadata.yml for backwards compatibility
+    metadata_path = client.renku_path.joinpath(OLD_METADATA_PATH)
+    with open(metadata_path, "w") as f:
+        f.write(
+            "# Dummy file kept for backwards compatibility, does not contain actual version\n"
+            "'http://schema.org/schemaVersion': '9'"
+        )
+
+    # NOTE: clone the repo
     communication.echo("Initializing new Renku repository... ")
     with client.lock:
         try:
@@ -311,11 +447,14 @@ def _init(
                 client=client,
                 name=name,
                 metadata=metadata,
+                custom_metadata=custom_metadata,
                 template_version=template_version,
                 immutable_template_files=template_data.get("immutable_template_files", []),
                 automated_update=template_data.get("allow_template_update", False),
                 force=force,
                 data_dir=data_dir,
+                description=description,
+                keywords=keywords,
             )
         except FileExistsError as e:
             raise errors.InvalidFileOperation(e)
@@ -323,7 +462,7 @@ def _init(
         communication.echo(
             "Project initialized.\n"
             f"You can undo this command by running 'git reset --hard {branch_name}'\n"
-            f"You can see changes made by running 'git diff {branch_name} {client.repo.head.ref.name}'"
+            f"You can see changes made by running 'git diff {branch_name} {client.repository.head.reference.name}'"
         )
     else:
         communication.echo("Project initialized.")
@@ -331,7 +470,7 @@ def _init(
 
 def init_command():
     """Init command builder."""
-    return Command().command(_init)
+    return Command().command(_init).with_database()
 
 
 def fetch_template_from_git(source, ref=None, tempdir=None):
@@ -342,39 +481,15 @@ def fetch_template_from_git(source, ref=None, tempdir=None):
     :param tempdir: temporary work directory path
     :return: tuple of (template folder, template version)
     """
-    if tempdir is None:
-        tempdir = Path(tempfile.mkdtemp())
-
-    try:
-        # clone the repo locally without checking out files
-        template_repo = git.Repo.clone_from(source, tempdir, no_checkout=True)
-    except git.exc.GitCommandError as e:
-        raise errors.GitError("Cannot clone repo from {0}".format(source)) from e
-
-    if ref:
-        try:
-            # fetch ref and set the HEAD
-            template_repo.remotes.origin.fetch()
-            try:
-                template_repo.head.reset(template_repo.commit(ref))
-            except git.exc.BadName:
-                ref = "origin/{0}".format(ref)
-                template_repo.head.reset(template_repo.commit(ref))
-            git_repo = git.Git(str(tempdir))
-        except (git.exc.GitCommandError, git.exc.BadName) as e:
-            raise errors.GitError("Cannot fetch and checkout reference {0}".format(ref)) from e
-    else:
-        template_repo.remotes.origin.fetch()
-        template_repo.head.reset(template_repo.commit())
-        git_repo = git.Git(str(tempdir))
+    template_repository = clone_repository(url=source, path=tempdir, checkout_revision=ref, install_lfs=False)
 
     # checkout the manifest
     try:
-        git_repo.checkout(TEMPLATE_MANIFEST)
-    except git.exc.GitCommandError as e:
-        raise errors.GitError("Cannot checkout manifest file {0}".format(TEMPLATE_MANIFEST)) from e
+        template_repository.checkout(TEMPLATE_MANIFEST)
+    except errors.GitCommandError as e:
+        raise errors.GitError(f"Cannot checkout manifest file {TEMPLATE_MANIFEST}") from e
 
-    return tempdir / TEMPLATE_MANIFEST, template_repo.head.commit.hexsha
+    return template_repository.path / TEMPLATE_MANIFEST, template_repository.head.commit.hexsha
 
 
 def fetch_template(template_source, template_ref):
@@ -393,13 +508,15 @@ def fetch_template(template_source, template_ref):
     else:
         from renku import __version__
 
-        if template_ref:
+        if template_ref and template_ref != "master":
             raise errors.ParameterError("Templates included in renku don't support specifying a template_ref")
 
-        template_folder = Path(pkg_resources.resource_filename("renku", "templates"))
+        ref = importlib_resources.files("renku") / "templates"
+        with importlib_resources.as_file(ref) as folder:
+            template_folder = folder
         template_manifest = read_template_manifest(template_folder)
         template_source = "renku"
-        template_version = __version__
+        template_version = str(__version__)
 
     return template_manifest, template_folder, template_source, template_version
 
@@ -438,6 +555,12 @@ def validate_template_manifest(manifest):
                 raise errors.InvalidTemplateError(
                     ('Template "{0}" doesn\'t have a {1} attribute'.format(template["name"], attribute))
                 )
+
+        if "variables" in template:
+            for key, variable in template["variables"].items():
+                if isinstance(variable, str):
+                    # NOTE: Backwards compatibility
+                    template["variables"][key] = {"description": variable}
     return True
 
 
@@ -447,6 +570,7 @@ def read_template_manifest(folder, checkout=False):
     :param folder: path where to find the template manifest file
     :param checkout: checkout the template folder from local repo
     """
+    folder = Path(folder)
     manifest_path = folder / TEMPLATE_MANIFEST
     try:
         manifest = yaml.safe_load(manifest_path.read_text())
@@ -455,18 +579,25 @@ def read_template_manifest(folder, checkout=False):
     validate_template_manifest(manifest)
 
     if checkout:
-        git_repo = git.Git(str(folder))
+        repository = Repository(folder)
         template_folders = [template["folder"] for template in manifest]
+        template_icons = [template["icon"] for template in manifest if "icon" in template]
         if len(template_folders) < 1:
             raise errors.InvalidTemplateError("Cannot find any valid template in manifest file")
         for template_folder in template_folders:
             template_path = folder / template_folder
             try:
-                git_repo.checkout(template_folder)
-            except git.exc.GitCommandError as e:
+                repository.checkout(template_folder)
+            except errors.GitCommandError as e:
                 raise errors.InvalidTemplateError('Cannot checkout the folder "{0}"'.format(template_folder)) from e
             validate_template(template_path)
 
+        for template_icon in template_icons:
+            try:
+                repository.checkout(template_icon)
+            except errors.GitCommandError as e:
+                raise errors.InvalidTemplateError('Cannot checkout the icon "{0}"'.format(template_icon)) from e
+
     return manifest
 
 
@@ -475,6 +606,7 @@ def create_from_template(
     client,
     name=None,
     metadata={},
+    custom_metadata=None,
     template_version=None,
     immutable_template_files=[],
     automated_update=False,
@@ -482,6 +614,8 @@ def create_from_template(
     data_dir=None,
     user=None,
     commit_message=None,
+    description=None,
+    keywords=None,
 ):
     """Initialize a new project from a template."""
 
@@ -489,11 +623,21 @@ def create_from_template(
 
     commit_only = [f"{RENKU_HOME}/"] + template_files
 
-    if "name" not in metadata:
+    if data_dir:
+        data_path = client.path / data_dir
+        data_path.mkdir(parents=True, exist_ok=True)
+        keep = data_path / ".gitkeep"
+        keep.touch(exist_ok=True)
+        commit_only.append(keep)
+
+    if "__name__" not in metadata:
         metadata["name"] = name
+        metadata["__name__"] = name
 
     with client.commit(commit_message=commit_message, commit_only=commit_only, skip_dirty_checks=True):
-        with client.with_metadata(name=name) as project:
+        with client.with_metadata(
+            name=name, description=description, custom_metadata=custom_metadata, keywords=keywords
+        ) as project:
             project.template_source = metadata["__template_source__"]
             project.template_ref = metadata["__template_ref__"]
             project.template_id = metadata["__template_id__"]
@@ -506,16 +650,15 @@ def create_from_template(
 
         if data_dir:
             client.set_value("renku", client.DATA_DIR_CONFIG_KEY, str(data_dir))
-            data_path = client.path / data_dir
-            data_path.mkdir(parents=True, exist_ok=True)
-            (data_path / ".gitkeep").touch(exist_ok=True)
 
 
+@inject.autoparams()
 def _create_from_template_local(
-    client,
     template_path,
     name,
+    client_dispatcher: IClientDispatcher,
     metadata={},
+    custom_metadata=None,
     default_metadata={},
     template_version=None,
     immutable_template_files=[],
@@ -525,34 +668,39 @@ def _create_from_template_local(
     ref=None,
     invoked_from=None,
     initial_branch=None,
+    commit_message=None,
+    description=None,
+    keywords=None,
 ):
-    """Initialize a new project from a template.
+    """Initialize a new project from a template."""
 
-    It creates a custom commit message and accepts custom user data.
-    """
-    command = "renku init" f' -n "{name}"' f' -s "{source}"' f' -r "{ref}"' f' -t "{template_path.name}"'
-    parameters = "".join([f' -p "{key}"="{value}"' for key, value in metadata.items()])
-    prefix = f"{invoked_from}: " if invoked_from else ""
-    commit_message = f"{prefix}{command}{parameters}"
+    client = client_dispatcher.current_client
 
     metadata = {**default_metadata, **metadata}
 
     client.init_repository(False, user, initial_branch=initial_branch)
 
+    # Initialize an empty database
+    database_gateway = inject.instance(IDatabaseGateway)
+    database_gateway.initialize()
+
     create_from_template(
         template_path=template_path,
         client=client,
         name=name,
         metadata=metadata,
+        custom_metadata=custom_metadata,
         template_version=template_version,
         immutable_template_files=immutable_template_files,
         automated_update=automated_template_update,
         force=False,
         user=user,
         commit_message=commit_message,
+        description=description,
+        keywords=keywords,
     )
 
 
 def create_from_template_local_command():
     """Command to initialize a new project from a template."""
-    return Command().command(_create_from_template_local)
+    return Command().command(_create_from_template_local).with_database()
