@@ -20,26 +20,28 @@
 import os
 from pathlib import Path
 
-import git
-
 from renku.core import errors
-from renku.core.incubation.command import Command
+from renku.core.management.command_builder import inject
+from renku.core.management.command_builder.command import Command
+from renku.core.management.interface.client_dispatcher import IClientDispatcher
 from renku.core.utils import communication
-from renku.core.utils.git import add_to_git
 
 
 def move_command():
     """Command to move or rename a file, a directory, or a symlink."""
-    return Command().command(_move).require_migration().require_clean().with_commit()
+    return Command().command(_move).require_migration().require_clean().with_database(write=True).with_commit()
 
 
-def _move(client, sources, destination, force, verbose, to_dataset):
+@inject.autoparams()
+def _move(sources, destination, force, verbose, to_dataset, client_dispatcher: IClientDispatcher):
     """Move files and check repository for potential problems."""
-    if to_dataset:
-        client.load_dataset(to_dataset, strict=True)
+    client = client_dispatcher.current_client
 
-    absolute_destination = _get_absolute_path(client, destination)
-    absolute_sources = [_get_absolute_path(client, src) for src in sources]
+    if to_dataset:
+        client.get_dataset(to_dataset, strict=True)
+
+    absolute_destination = _get_absolute_path(destination)
+    absolute_sources = [_get_absolute_path(src) for src in sources]
 
     is_rename = len(absolute_sources) == 1 and (
         not absolute_destination.exists() or (absolute_destination.is_file() and absolute_sources[0].is_file())
@@ -60,7 +62,7 @@ def _move(client, sources, destination, force, verbose, to_dataset):
     # NOTE: we don't check if source and destination are the same or if multiple sources are moved to the same
     # destination; git mv will check those and we raise if git mv fails.
 
-    _warn_about_ignored_destinations(client, files.values())
+    _warn_about_ignored_destinations(files.values())
 
     if not absolute_destination.exists() and not absolute_destination.is_symlink():
         if is_rename:
@@ -69,8 +71,8 @@ def _move(client, sources, destination, force, verbose, to_dataset):
             absolute_destination.mkdir(parents=True, exist_ok=True)
 
     try:
-        client.repo.git.mv([str(src) for src in sources] + [str(destination)], force=force)
-    except git.GitError as e:
+        client.repository.move(*sources, destination=destination, force=force)
+    except errors.GitCommandError as e:
         raise errors.OperationError(f"Git operation failed: {e}")
 
     # Handle symlinks
@@ -83,17 +85,16 @@ def _move(client, sources, destination, force, verbose, to_dataset):
     files_to_untrack = (str(src.relative_to(client.path)) for src in files)
     client.untrack_paths_from_storage(*files_to_untrack)
     # NOTE: Warn about filter after untracking from LFS to avoid warning about LFS filters
-    _warn_about_git_filters(client, files)
+    _warn_about_git_filters(files)
     client.track_paths_in_storage(*[dst for dst in files.values() if not dst.is_dir()])
 
     # NOTE: Force-add to include possible ignored files
-    add_to_git(client.repo.git, *files.values(), force=True)
-    commit = client.repo.index.commit(f"renku mv: committing {len(files)} moved files")
+    client.repository.add(*files.values(), force=True)
 
-    client.move_files(files=files, to_dataset=to_dataset, commit=commit)
+    client.move_files(files=files, to_dataset=to_dataset)
 
     if verbose:
-        _show_moved_files(client, files)
+        _show_moved_files(client.path, files)
 
 
 def _traverse_path(path):
@@ -111,8 +112,11 @@ def _get_dst(path, src_root, dst_root, is_rename):
     return dst_root / parent / os.path.relpath(path, start=src_root)
 
 
-def _get_absolute_path(client, path):
+@inject.autoparams()
+def _get_absolute_path(path, client_dispatcher: IClientDispatcher):
     """Resolve path and raise if path is outside the repo or is protected."""
+    client = client_dispatcher.current_client
+
     abs_path = Path(os.path.abspath(path))
 
     if client.is_protected_path(abs_path):
@@ -139,19 +143,25 @@ def _check_existing_destinations(destinations):
     raise errors.ParameterError(f"The following move target exist, use '--force' flag to overwrite them:\n\t{existing}")
 
 
-def _warn_about_ignored_destinations(client, destinations):
+@inject.autoparams()
+def _warn_about_ignored_destinations(destinations, client_dispatcher: IClientDispatcher):
+    client = client_dispatcher.current_client
+
     ignored = client.find_ignored_paths(*destinations)
     if ignored:
         ignored = "\n\t".join((str(Path(p).relative_to(client.path)) for p in ignored))
         communication.warn(f"The following moved path match .gitignore:\n\t{ignored}")
 
 
-def _warn_about_git_filters(client, files):
+@inject.autoparams()
+def _warn_about_git_filters(files, client_dispatcher: IClientDispatcher):
     """Check if there are any git attributes for files including LFS."""
+    client = client_dispatcher.current_client
+
     src_attrs = []
     dst_attrs = []
 
-    for path, attrs in client.find_attr(*files).items():
+    for path, attrs in client.repository.get_attributes(*files).items():
         src = Path(path)
         dst = files[src].relative_to(client.path)
         src = src.relative_to(client.path)
@@ -176,8 +186,8 @@ def _warn_about_git_filters(client, files):
         )
 
 
-def _show_moved_files(client, files):
+def _show_moved_files(client_path, files):
     for path in sorted(files):
-        src = path.relative_to(client.path)
-        dst = files[path].relative_to(client.path)
+        src = path.relative_to(client_path)
+        dst = files[path].relative_to(client_path)
         communication.echo(f"{src} -> {dst}")
