@@ -22,23 +22,29 @@ import re
 import urllib
 from pathlib import Path
 from string import Template
+from typing import TYPE_CHECKING, List
 from urllib import parse as urlparse
 
 import attr
-import requests
-from marshmallow import pre_load
 from tqdm import tqdm
 
 from renku.core import errors
 from renku.core.commands.providers.api import ExporterApi, ProviderApi
+from renku.core.commands.providers.dataverse_metadata_templates import (
+    AUTHOR_METADATA_TEMPLATE,
+    CONTACT_METADATA_TEMPLATE,
+    DATASET_METADATA_TEMPLATE,
+)
 from renku.core.commands.providers.doi import DOIProvider
-from renku.core.models.datasets import Dataset, DatasetFile, DatasetSchema
-from renku.core.models.provenance.agents import PersonSchema
+from renku.core.management.command_builder import inject
+from renku.core.management.interface.client_dispatcher import IClientDispatcher
+from renku.core.metadata.immutable import DynamicProxy
 from renku.core.utils import communication
 from renku.core.utils.doi import extract_doi, is_doi
-from renku.core.utils.requests import retry
+from renku.core.utils.file_size import bytes_to_unit
 
-from .dataverse_metadata_templates import AUTHOR_METADATA_TEMPLATE, CONTACT_METADATA_TEMPLATE, DATASET_METADATA_TEMPLATE
+if TYPE_CHECKING:
+    from renku.core.commands.providers.models import ProviderDataset
 
 DATAVERSE_API_PATH = "api/v1"
 
@@ -48,7 +54,6 @@ DATAVERSE_VERSIONS_API = "datasets/:persistentId/versions"
 DATAVERSE_FILE_API = "access/datafile/:persistentId/"
 DATAVERSE_EXPORTER = "schema.org"
 
-
 DATAVERSE_SUBJECTS = [
     "Agricultural Sciences",
     "Arts and Humanities",
@@ -67,57 +72,32 @@ DATAVERSE_SUBJECTS = [
 ]
 
 
-class _DataverseDatasetSchema(DatasetSchema):
-    """Schema for Dataverse datasets."""
-
-    @pre_load
-    def fix_data(self, data, **kwargs):
-        """Fix data that is received from Dataverse."""
-        # Fix context
-        context = data.get("@context")
-        if context and isinstance(context, str):
-            if context == "http://schema.org":
-                context = "http://schema.org/"
-            data["@context"] = {"@base": context, "@vocab": context}
-
-        # Add type to creators
-        creators = data.get("creator", [])
-        for c in creators:
-            c["@type"] = [str(t) for t in PersonSchema.opts.rdf_type]
-
-        # Fix license to be a string
-        license = data.get("license")
-        if license and isinstance(license, dict):
-            data["license"] = license.get("url", "")
-
-        return data
-
-
 def check_dataverse_uri(url):
     """Check if an URL points to a dataverse instance."""
+    from renku.core.utils import requests
+
     url_parts = list(urlparse.urlparse(url))
     url_parts[2] = pathlib.posixpath.join(DATAVERSE_API_PATH, DATAVERSE_VERSION_API)
 
     url_parts[3:6] = [""] * 3
     version_url = urlparse.urlunparse(url_parts)
 
-    with retry() as session:
-        response = session.get(version_url)
+    response = requests.get(version_url)
 
-        if response.status_code != 200:
-            return False
+    if response.status_code != 200:
+        return False
 
-        version_data = response.json()
+    version_data = response.json()
 
-        if "status" not in version_data or "data" not in version_data:
-            return False
+    if "status" not in version_data or "data" not in version_data:
+        return False
 
-        version_info = version_data["data"]
+    version_info = version_data["data"]
 
-        if "version" not in version_info or "build" not in version_info:
-            return False
+    if "version" not in version_info or "build" not in version_info:
+        return False
 
-        return True
+    return True
 
 
 def check_dataverse_doi(doi):
@@ -205,13 +185,14 @@ class DataverseProvider(ProviderApi):
 
     def _make_request(self, uri):
         """Execute network request."""
-        with retry() as session:
-            response = session.get(uri, headers={"Accept": self._accept})
-            if response.status_code != 200:
-                raise LookupError("record not found. Status: {}".format(response.status_code))
-            return response
+        from renku.core.utils import requests
 
-    def find_record(self, uri, client=None, **kwargs):
+        response = requests.get(uri, headers={"Accept": self._accept})
+        if response.status_code != 200:
+            raise LookupError("record not found. Status: {}".format(response.status_code))
+        return response
+
+    def find_record(self, uri, **kwargs):
         """Retrieves a record from Dataverse.
 
         :raises: ``LookupError``
@@ -244,10 +225,13 @@ class DataverseProvider(ProviderApi):
             dataset=dataset, access_token=access_token, server_url=self._server_url, dataverse_name=self._dataverse_name
         )
 
-    def set_parameters(self, client, *, dataverse_server, dataverse_name, **kwargs):
+    @inject.autoparams()
+    def set_parameters(self, client_dispatcher: IClientDispatcher, *, dataverse_server, dataverse_name, **kwargs):
         """Set and validate required parameters for a provider."""
         CONFIG_BASE_URL = "server_url"
 
+        client = client_dispatcher.current_client
+
         if not dataverse_server:
             dataverse_server = client.get_value("dataverse", CONFIG_BASE_URL)
         else:
@@ -273,6 +257,8 @@ class DataverseRecordSerializer:
 
     _json = attr.ib(default=None, kw_only=True)
 
+    _files_info = attr.ib(default=None, kw_only=True)
+
     def is_last_version(self, uri):
         """Check if record is at last possible version."""
         return True
@@ -283,6 +269,14 @@ class DataverseRecordSerializer:
         property_name = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", property_name)
         return re.sub("([a-z0-9])([A-Z])", r"\1_\2", property_name).lower()
 
+    @property
+    def files_info(self) -> List[DynamicProxy]:
+        """Return list of dataset file proxies.
+
+        NOTE: This is only valid after a call to ``as_dataset``.
+        """
+        return self._files_info
+
     @property
     def files(self):
         """Get all file metadata entries."""
@@ -309,15 +303,48 @@ class DataverseRecordSerializer:
 
     def get_files(self):
         """Get Dataverse files metadata as ``DataverseFileSerializer``."""
-        if not self.files:
+        files = self.files
+        if not files:
             raise LookupError("no files have been found - deposit is empty or protected")
 
-        return [DataverseFileSerializer(**file_) for file_ in self.files]
+        return [DataverseFileSerializer(**file_) for file_ in files]
+
+    def as_dataset(self, client) -> "ProviderDataset":
+        """Deserialize ``DataverseRecordSerializer`` to ``Dataset``."""
+
+        from marshmallow import pre_load
+
+        from renku.core.commands.providers.models import ProviderDataset, ProviderDatasetSchema
+        from renku.core.commands.schema.agent import PersonSchema
+        from renku.core.models.dataset import DatasetFile
+
+        class _DataverseDatasetSchema(ProviderDatasetSchema):
+            """Schema for Dataverse datasets."""
+
+            @pre_load
+            def fix_data(self, data, **kwargs):
+                """Fix data that is received from Dataverse."""
+                # Fix context
+                context = data.get("@context")
+                if context and isinstance(context, str):
+                    if context == "http://schema.org":
+                        context = "http://schema.org/"
+                    data["@context"] = {"@base": context, "@vocab": context}
+
+                # Add type to creators
+                creators = data.get("creator", [])
+                for c in creators:
+                    c["@type"] = [str(t) for t in PersonSchema.opts.rdf_type]
+
+                # Fix license to be a string
+                license = data.get("license")
+                if license and isinstance(license, dict):
+                    data["license"] = license.get("url", "")
+
+                return data
 
-    def as_dataset(self, client):
-        """Deserialize `DataverseRecordSerializer` to `Dataset`."""
         files = self.get_files()
-        dataset = Dataset.from_jsonld(self._json, client=client, schema_class=_DataverseDatasetSchema)
+        dataset = ProviderDataset.from_jsonld(data=self._json, schema_class=_DataverseDatasetSchema)
 
         dataset.version = self.version
 
@@ -328,20 +355,22 @@ class DataverseRecordSerializer:
             if creator.affiliation == "":
                 creator.affiliation = None
 
-        serialized_files = []
-        for file_ in files:
-            remote_ = file_.remote_url
-            dataset_file = DatasetFile(
-                source=remote_.geturl(),
-                id=file_._id if file_._id else file_.name,
-                filename=file_.name,
-                filesize=file_.content_size,
-                filetype=file_.file_format,
-                path="",
-            )
-            serialized_files.append(dataset_file)
+        dataset_files = []
+        files_info = []
+        for file in files:
+            remote = file.remote_url
+            dataset_file = DatasetFile(entity=None, source=remote.geturl())
+            file_info = DynamicProxy(dataset_file)
+            file_info.filename = Path(file.name).name
+            file_info.checksum = ""
+            file_info.size_in_mb = bytes_to_unit(file.content_size, "mi")
+            file_info.filetype = file.file_format
 
-        dataset.files = serialized_files
+            dataset_files.append(dataset_file)
+            files_info.append(file_info)
+
+        dataset.dataset_files = dataset_files
+        self._files_info = files_info
 
         return dataset
 
@@ -407,7 +436,7 @@ class DataverseExporter(ExporterApi):
         """Endpoint for creation of access token."""
         return urllib.parse.urljoin(self._server_url, "/dataverseuser.xhtml?selectTab=apiTokenTab")
 
-    def export(self, publish, **kwargs):
+    def export(self, publish, client=None, **kwargs):
         """Execute export process."""
         deposition = _DataverseDeposition(server_url=self._server_url, access_token=self.access_token)
         metadata = self._get_dataset_metadata()
@@ -415,12 +444,13 @@ class DataverseExporter(ExporterApi):
         dataset_pid = response.json()["data"]["persistentId"]
 
         with tqdm(total=len(self.dataset.files)) as progressbar:
-            for file_ in self.dataset.files:
+            for file in self.dataset.files:
                 try:
-                    path = Path(file_.path).relative_to(self.dataset.data_dir)
+                    path = (client.path / file.entity.path).relative_to(self.dataset.data_dir)
                 except ValueError:
-                    path = Path(file_.path)
-                deposition.upload_file(full_path=file_.full_path, path_in_dataset=path)
+                    path = Path(file.entity.path)
+                filepath = client.repository.copy_content_to_file(path=file.entity.path, checksum=file.entity.checksum)
+                deposition.upload_file(full_path=filepath, path_in_dataset=path)
                 progressbar.update(1)
 
         if publish:
@@ -535,11 +565,12 @@ class _DataverseDeposition:
         return urllib.parse.urlunparse(url_parts)
 
     def _post(self, url, json=None, data=None, files=None):
+        from renku.core.utils import requests
+
         headers = {"X-Dataverse-key": self.access_token}
         try:
-            with retry() as session:
-                return session.post(url=url, json=json, data=data, files=files, headers=headers)
-        except requests.exceptions.RequestException as e:
+            return requests.post(url=url, json=json, data=data, files=files, headers=headers)
+        except errors.RequestError as e:
             raise errors.ExportError("Cannot POST to remote server.") from e
 
     @staticmethod
