@@ -24,12 +24,12 @@ from urllib import parse as urlparse
 from uuid import UUID, uuid4
 
 import attr
-import requests
 
 from renku.core import errors
 from renku.core.commands.providers.api import ExporterApi, ProviderApi
+from renku.core.management.command_builder import inject
+from renku.core.management.interface.client_dispatcher import IClientDispatcher
 from renku.core.utils import communication
-from renku.core.utils.requests import retry
 
 
 @attr.s
@@ -63,10 +63,13 @@ class OLOSProvider(ProviderApi):
         """Create export manager for given dataset."""
         return OLOSExporter(dataset=dataset, access_token=access_token, server_url=self._server_url)
 
-    def set_parameters(self, client, *, dlcm_server=None, **kwargs):
+    @inject.autoparams()
+    def set_parameters(self, client_dispatcher: IClientDispatcher, *, dlcm_server=None, **kwargs):
         """Set and validate required parameters for a provider."""
         config_base_url = "server_url"
 
+        client = client_dispatcher.current_client
+
         if not dlcm_server:
             dlcm_server = client.get_value("olos", config_base_url)
         else:
@@ -96,7 +99,7 @@ class OLOSExporter(ExporterApi):
         """Endpoint for creation of access token."""
         return urllib.parse.urljoin(self._server_url, "/portal by clicking on the top-right menu and selecting 'token'")
 
-    def export(self, publish, **kwargs):
+    def export(self, publish, client=None, **kwargs):
         """Execute export process."""
         deposition = _OLOSDeposition(server_url=self._server_url, access_token=self.access_token)
 
@@ -108,12 +111,13 @@ class OLOSExporter(ExporterApi):
         communication.start_progress(progress_text, total=len(self.dataset.files))
 
         try:
-            for file_ in self.dataset.files:
+            for file in self.dataset.files:
                 try:
-                    path = Path(file_.path).relative_to(self.dataset.data_dir)
+                    path = (client.path / file.entity.path).relative_to(self.dataset.data_dir)
                 except ValueError:
-                    path = Path(file_.path)
-                deposition.upload_file(full_path=file_.full_path, path_in_dataset=path)
+                    path = Path(file.entity.path)
+                filepath = client.repository.copy_content_to_file(path=file.entity.path, checksum=file.entity.checksum)
+                deposition.upload_file(full_path=filepath, path_in_dataset=path)
                 communication.update_progress(progress_text, amount=1)
         finally:
             communication.finalize_progress(progress_text)
@@ -124,7 +128,7 @@ class OLOSExporter(ExporterApi):
         try:
             identifier = UUID(self.dataset.identifier, version=4)
         except ValueError:
-            identifier = uuid4()
+            identifier = uuid4().hex
         metadata = {
             "publicationDate": datetime.date.today().isoformat(),
             "description": self.dataset.description,
@@ -235,20 +239,22 @@ class _OLOSDeposition:
         url_parts = url_parts._replace(path=api_path, query=query_params)
         return urllib.parse.urlunparse(url_parts)
 
-    def _get(self, url, json=None, data=None, files=None):
+    def _get(self, url):
+        from renku.core.utils import requests
+
         headers = {"Authorization": f"Bearer {self.access_token}"}
         try:
-            with retry() as session:
-                return session.get(url=url, json=json, data=data, files=files, headers=headers)
-        except requests.exceptions.RequestException as e:
-            raise errors.ExportError("Cannot GET to remote server.") from e
+            return requests.get(url=url, headers=headers)
+        except errors.RequestError as e:
+            raise errors.ExportError("Cannot GET from remote server.") from e
 
     def _post(self, url, json=None, data=None, files=None):
+        from renku.core.utils import requests
+
         headers = {"Authorization": f"Bearer {self.access_token}"}
         try:
-            with retry() as session:
-                return session.post(url=url, json=json, data=data, files=files, headers=headers)
-        except requests.exceptions.RequestException as e:
+            return requests.post(url=url, json=json, data=data, files=files, headers=headers)
+        except errors.RequestError as e:
             raise errors.ExportError("Cannot POST to remote server.") from e
 
     @staticmethod
