@@ -23,17 +23,19 @@ import shutil
 import urllib
 from pathlib import Path
 from subprocess import PIPE, SubprocessError, run
+from typing import List
 
 import attr
-import requests
 
-from renku import LocalClient
 from renku.core import errors
 from renku.core.commands.login import read_renku_token
 from renku.core.commands.providers.api import ProviderApi
-from renku.core.management.migrate import is_project_unsupported, migrate
+from renku.core.management.command_builder.command import inject
+from renku.core.management.interface.client_dispatcher import IClientDispatcher
+from renku.core.management.interface.database_dispatcher import IDatabaseDispatcher
+from renku.core.metadata.immutable import DynamicProxy
 from renku.core.utils import communication
-from renku.core.utils.migrate import MigrationType
+from renku.core.utils.git import clone_renku_repository, get_cache_directory_for_repository
 
 
 @attr.s
@@ -63,7 +65,7 @@ class RenkuProvider(ProviderApi):
         """Whether this provider supports dataset import."""
         return True
 
-    def find_record(self, uri, client=None, **kwargs):
+    def find_record(self, uri, **kwargs):
         """Retrieves a dataset from Renku.
 
         :raises: ``NotFound``, ``OperationError``, ``ParameterError``
@@ -73,7 +75,7 @@ class RenkuProvider(ProviderApi):
         self._uri = uri
         self._gitlab_token = kwargs.get("gitlab_token")
 
-        self._prepare_auth(client, uri)
+        self._prepare_auth(uri)
 
         name, identifier, latest_version_uri, kg_url = self._fetch_dataset_info(uri)
 
@@ -169,13 +171,15 @@ class RenkuProvider(ProviderApi):
         return project_id, dataset_name_or_id
 
     def _query_knowledge_graph(self, url):
+        from renku.core.utils import requests
+
         if self._renku_token and not self._gitlab_token:
             # NOTE: Authorization with renku token requires going through the gateway route
             url = url.replace("/knowledge-graph/", "/api/kg/")
 
         try:
             response = requests.get(url, headers=self._authorization_header)
-        except urllib.error.HTTPError as e:
+        except errors.RequestError as e:
             raise errors.OperationError(f"Cannot access knowledge graph: {url}") from e
 
         if response.status_code == 404:
@@ -197,11 +201,11 @@ class RenkuProvider(ProviderApi):
 
         return urls.get("ssh"), urls.get("http")
 
-    def _prepare_auth(self, client, uri):
+    def _prepare_auth(self, uri):
         if self._gitlab_token:
             token = self._gitlab_token
         else:
-            self._renku_token = read_renku_token(client, endpoint=uri)
+            self._renku_token = read_renku_token(endpoint=uri)
             token = self._renku_token
 
         self._authorization_header = {"Authorization": f"Bearer {token}"} if token else {}
@@ -227,6 +231,7 @@ class _RenkuRecordSerializer:
         self._project_url = None
         self._project_repo = None
         self._remote_client = None
+        self._files_info = []
 
     @staticmethod
     def _get_file_size(remote_client, path):
@@ -266,15 +271,26 @@ class _RenkuRecordSerializer:
 
         # Return size of the file on disk
         full_path = remote_client.path / path
-        return os.path.getsize(full_path)
+        return float(os.path.getsize(full_path))
+
+    @property
+    def files_info(self) -> List[DynamicProxy]:
+        """Return list of dataset file proxies.
+
+        NOTE: This is only valid after a call to ``as_dataset``.
+        """
+        return self._files_info
 
     def as_dataset(self, client):
         """Return encapsulated dataset instance."""
-        self._fetch_dataset(client)
+        self._fetch_dataset()
         return self._dataset
 
-    def import_images(self, client, dataset):
+    @inject.autoparams()
+    def import_images(self, dataset, client_dispatcher: IClientDispatcher):
         """Add images from remote dataset."""
+        client = client_dispatcher.current_client
+
         if not self._dataset.images:
             return
 
@@ -288,7 +304,7 @@ class _RenkuRecordSerializer:
 
             shutil.copy(remote_image_path, local_image_path)
 
-        dataset.images = self._dataset.images
+        dataset.images = self._dataset.images or []
 
     def is_last_version(self, uri):
         """Check if dataset is at last possible version."""
@@ -301,7 +317,7 @@ class _RenkuRecordSerializer:
 
     @property
     def repository(self):
-        """The cloned repo that contains the dataset."""
+        """The cloned repository that contains the dataset."""
         return self._project_repo
 
     @staticmethod
@@ -325,9 +341,13 @@ class _RenkuRecordSerializer:
         """Whether the dataset datadir exists (might be missing in git if empty)."""
         return (self._remote_client.path / self._dataset.data_dir).exists()
 
-    def _fetch_dataset(self, client):
-        repo_path = None
-        repo = None
+    @inject.autoparams()
+    def _fetch_dataset(self, client_dispatcher: IClientDispatcher, database_dispatcher: IDatabaseDispatcher):
+        from renku.core.management.client import LocalClient
+        from renku.core.models.dataset import get_dataset_data_dir
+
+        repository = None
+        client = client_dispatcher.current_client
 
         parsed_uri = urllib.parse.urlparse(self._uri)
 
@@ -335,11 +355,14 @@ class _RenkuRecordSerializer:
         # Clone the project
         for url in urls:
             try:
-                repo, repo_path = client.prepare_git_repo(
+                repository = clone_renku_repository(
                     url=url,
+                    path=get_cache_directory_for_repository(client=client, url=url),
                     gitlab_token=self._gitlab_token,
-                    renku_token=self._renku_token,
                     deployment_hostname=parsed_uri.netloc,
+                    depth=None,
+                    reuse_existing_repository=True,
+                    use_renku_credentials=True,
                 )
             except errors.GitError:
                 pass
@@ -350,28 +373,48 @@ class _RenkuRecordSerializer:
         if not self._project_url:
             raise errors.ParameterError("Cannot clone remote projects:\n\t" + "\n\t".join(urls), param_hint=self._uri)
 
-        self._remote_client = LocalClient(repo_path)
-        self._migrate_project(self._remote_client)
-        self._project_repo = repo
+        self._remote_client = LocalClient(repository.path)
+        client_dispatcher.push_created_client_to_stack(self._remote_client)
+        database_dispatcher.push_database_to_stack(self._remote_client.database_path)
 
-        self._dataset = self._remote_client.load_dataset(self._name)
+        try:
+            self._migrate_project()
+            self._project_repo = repository
+
+            self._dataset = self._remote_client.get_dataset(self._name)
+        finally:
+            database_dispatcher.pop_database()
+            client_dispatcher.pop_client()
 
         if not self._dataset:
             raise errors.ParameterError(f"Cannot find dataset '{self._name}' in project '{self._project_url}'")
 
-        for file_ in self._dataset.files:
-            file_.checksum = self._remote_client.repo.git.hash_object(file_.path)
-            file_.filesize = _RenkuRecordSerializer._get_file_size(self._remote_client, file_.path)
-            file_.filetype = Path(file_.path).suffix.replace(".", "")
+        self._dataset.data_dir = get_dataset_data_dir(self._remote_client, self._dataset)
+
+        files_info = []
+        for file in self._dataset.files:
+            file_info = DynamicProxy(file)
+            file_info.checksum = file.entity.checksum
+            file_info.filename = Path(file.entity.path).name
+            file_info.size_in_mb = _RenkuRecordSerializer._get_file_size(self._remote_client, file.entity.path)
+            file_info.filetype = Path(file.entity.path).suffix.replace(".", "")
+            files_info.append(file_info)
+
+        self._files_info = files_info
 
     @staticmethod
-    def _migrate_project(client):
-        if is_project_unsupported(client):
+    def _migrate_project():
+        from renku.core.management.migrate import is_project_unsupported, migrate  # Slow import
+        from renku.core.management.migrations.utils import MigrationType
+
+        if is_project_unsupported():
             return
-        # NOTE: We are not interested in migrating workflows when importing datasets
-        client.migration_type = ~MigrationType.WORKFLOWS
+
         try:
             communication.disable()
-            migrate(client, skip_template_update=True, skip_docker_update=True)
+            # NOTE: We are not interested in migrating workflows when importing datasets
+            migrate(
+                skip_template_update=True, skip_docker_update=True, migration_type=~MigrationType.WORKFLOWS, strict=True
+            )
         finally:
             communication.enable()
