@@ -16,24 +16,23 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Zenodo API integration."""
+
 import json
 import os
 import pathlib
 import urllib
 from pathlib import Path
+from typing import List
 from urllib.parse import urlparse
 
 import attr
-import requests
-from marshmallow import pre_load
 from tqdm import tqdm
 
-from renku.core import errors
 from renku.core.commands.providers.api import ExporterApi, ProviderApi
 from renku.core.commands.providers.doi import DOIProvider
-from renku.core.models.datasets import Dataset, DatasetFile, DatasetSchema
-from renku.core.models.provenance.agents import PersonSchema
-from renku.core.utils.requests import retry
+from renku.core.commands.schema.dataset import dump_dataset_as_jsonld
+from renku.core.metadata.immutable import DynamicProxy
+from renku.core.utils.file_size import bytes_to_unit
 
 ZENODO_BASE_URL = "https://zenodo.org"
 ZENODO_SANDBOX_URL = "https://sandbox.zenodo.org/"
@@ -49,60 +48,11 @@ ZENODO_FILES_URL = "depositions/{0}/files"
 ZENODO_NEW_DEPOSIT_URL = "depositions"
 
 
-class _ZenodoDatasetSchema(DatasetSchema):
-    """Schema for Dataverse datasets."""
-
-    @pre_load
-    def fix_data(self, data, **kwargs):
-        """Fix data that is received from Dataverse."""
-        # Fix context
-        context = data.get("@context")
-        if context and isinstance(context, str):
-            if context == "https://schema.org/":
-                context = "http://schema.org/"
-            data["@context"] = {"@base": context, "@vocab": context}
-
-        # Add type to creators
-        creators = data.get("creator", [])
-        for c in creators:
-            c["@type"] = [str(t) for t in PersonSchema.opts.rdf_type]
-
-        # Fix license to be a string
-        license = data.get("license")
-        if license and isinstance(license, dict):
-            data["license"] = license.get("url", "")
-
-        # Delete existing isPartOf
-        data.pop("isPartOf", None)
-
-        return data
-
-
 def make_records_url(record_id):
     """Create URL to access record by ID."""
     return urllib.parse.urljoin(ZENODO_BASE_URL, pathlib.posixpath.join(ZENODO_API_PATH, "records", record_id))
 
 
-def check_or_raise(response):
-    """Check for expected response status code."""
-    if response.status_code not in [200, 201, 202]:
-        if response.status_code == 401:
-            raise errors.AuthenticationError("Access unauthorized - update access token.")
-
-        if response.status_code == 400:
-            err_response = response.json()
-            messages = [
-                '"{0}" failed with "{1}"'.format(err["field"], err["message"]) for err in err_response["errors"]
-            ]
-
-            raise errors.ExportError(
-                "\n" + "\n".join(messages) + "\nSee `renku dataset edit -h` for details on how to edit" " metadata"
-            )
-
-        else:
-            raise errors.ExportError(response.content)
-
-
 @attr.s
 class ZenodoFileSerializer:
     """Zenodo record file."""
@@ -235,6 +185,8 @@ class ZenodoRecordSerializer:
 
     _uri = attr.ib(default=None, kw_only=True)
 
+    _files_info = attr.ib(default=None, kw_only=True)
+
     @property
     def version(self):
         """Get record version."""
@@ -245,6 +197,14 @@ class ZenodoRecordSerializer:
         """Get uri of latest version."""
         return self.links.get("latest_html")
 
+    @property
+    def files_info(self) -> List[DynamicProxy]:
+        """Return list of dataset file proxies.
+
+        NOTE: This is only valid after a call to ``as_dataset``.
+        """
+        return self._files_info
+
     def is_last_version(self, uri):
         """Check if this record is the latest version."""
         return ZenodoProvider.record_id(self.links.get("latest_html")) == self.record_id
@@ -273,29 +233,61 @@ class ZenodoRecordSerializer:
 
     def as_dataset(self, client):
         """Deserialize `ZenodoRecordSerializer` to `Dataset`."""
+        from marshmallow import pre_load
+
+        from renku.core.commands.providers.models import ProviderDataset, ProviderDatasetSchema
+        from renku.core.commands.schema.agent import PersonSchema
+        from renku.core.models.dataset import DatasetFile
+
+        class _ZenodoDatasetSchema(ProviderDatasetSchema):
+            """Schema for Dataverse datasets."""
+
+            @pre_load
+            def fix_data(self, data, **kwargs):
+                """Fix data that is received from Dataverse."""
+                # Fix context
+                context = data.get("@context")
+                if context and isinstance(context, str):
+                    if context == "https://schema.org/":
+                        context = "http://schema.org/"
+                    data["@context"] = {"@base": context, "@vocab": context}
+
+                # Add type to creators
+                creators = data.get("creator", [])
+                for c in creators:
+                    c["@type"] = [str(t) for t in PersonSchema.opts.rdf_type]
+
+                # Fix license to be a string
+                license = data.get("license")
+                if license and isinstance(license, dict):
+                    data["license"] = license.get("url", "")
+
+                # Delete existing isPartOf
+                data.pop("isPartOf", None)
+                data.pop("sameAs", None)
+
+                return data
+
         files = self.get_files()
         metadata = self.get_jsonld()
-        dataset = Dataset.from_jsonld(metadata, client=client, schema_class=_ZenodoDatasetSchema)
+        dataset = ProviderDataset.from_jsonld(metadata, schema_class=_ZenodoDatasetSchema)
 
-        serialized_files = []
-        for file_ in files:
-            remote_ = file_.remote_url
-            dataset_file = DatasetFile(
-                source=remote_.geturl(),
-                id=file_.id,
-                checksum=file_.checksum,
-                filename=file_.filename,
-                filesize=file_.filesize,
-                filetype=file_.type,
-                path="",
-            )
-            serialized_files.append(dataset_file)
+        dataset_files = []
+        files_info = []
+        for file in files:
+            remote = file.remote_url
+            dataset_file = DatasetFile(entity=None, source=remote.geturl())
+            file_info = DynamicProxy(dataset_file)
+            file_info.checksum = file.checksum
+            file_info.filename = Path(file.filename).name
+            file_info.size_in_mb = bytes_to_unit(file.filesize, "mi")
+            file_info.filetype = file.type
 
-        dataset.files = serialized_files
+            dataset_files.append(dataset_file)
+            files_info.append(file_info)
 
-        if isinstance(dataset.url, dict) and "_id" in dataset.url:
-            dataset.url = urllib.parse.urlparse(dataset.url.pop("_id"))
-            dataset.url = dataset.url.geturl()
+        dataset.dataset_files = dataset_files
+        self._files_info = files_info
 
         return dataset
 
@@ -358,27 +350,32 @@ class ZenodoDeposition:
 
     def new_deposition(self):
         """Create new deposition on Zenodo."""
+        from renku.core.utils import requests
+
         response = requests.post(
             url=self.new_deposit_url, params=self.exporter.default_params, json={}, headers=self.exporter.HEADERS
         )
-        check_or_raise(response)
+        requests.check_response(response)
 
         return response
 
-    def upload_file(self, filepath):
+    def upload_file(self, filepath, path_in_repo):
         """Upload and attach a file to existing deposition on Zenodo."""
-        request_payload = {"filename": Path(filepath).name}
-        file = {"file": open(str(filepath), "rb")}
+        from renku.core.utils import requests
 
+        request_payload = {"filename": Path(path_in_repo).name}
+        file = {"file": (Path(path_in_repo).name, open(str(filepath), "rb"))}
         response = requests.post(
             url=self.upload_file_url, params=self.exporter.default_params, data=request_payload, files=file
         )
-        check_or_raise(response)
+        requests.check_response(response)
 
         return response
 
     def attach_metadata(self, dataset, tag):
         """Attach metadata to deposition on Zenodo."""
+        from renku.core.utils import requests
+
         request_payload = {
             "metadata": {
                 "title": dataset.title,
@@ -402,14 +399,16 @@ class ZenodoDeposition:
             data=json.dumps(request_payload),
             headers=self.exporter.HEADERS,
         )
-        check_or_raise(response)
+        requests.check_response(response)
 
         return response
 
     def publish_deposition(self, secret):
         """Publish existing deposition."""
+        from renku.core.utils import requests
+
         response = requests.post(url=self.publish_url, params=self.exporter.default_params)
-        check_or_raise(response)
+        requests.check_response(response)
 
         return response
 
@@ -451,11 +450,11 @@ class ZenodoExporter(ExporterApi):
 
     def dataset_to_request(self):
         """Prepare dataset metadata for request."""
-        jsonld = self.dataset.as_jsonld()
+        jsonld = dump_dataset_as_jsonld(self.dataset)
         jsonld["upload_type"] = "dataset"
         return jsonld
 
-    def export(self, publish, tag=None, **kwargs):
+    def export(self, publish, tag=None, client=None, **kwargs):
         """Execute entire export process."""
         # Step 1. Create new deposition
         deposition = ZenodoDeposition(exporter=self)
@@ -465,8 +464,9 @@ class ZenodoExporter(ExporterApi):
 
         # Step 3. Upload all files to created deposition
         with tqdm(total=len(self.dataset.files)) as progressbar:
-            for file_ in self.dataset.files:
-                deposition.upload_file(file_.full_path)
+            for file in self.dataset.files:
+                filepath = client.repository.copy_content_to_file(path=file.entity.path, checksum=file.entity.checksum)
+                deposition.upload_file(filepath, path_in_repo=file.entity.path)
                 progressbar.update(1)
 
         # Step 4. Publish newly created deposition
@@ -519,13 +519,14 @@ class ZenodoProvider(ProviderApi):
 
     def make_request(self, uri):
         """Execute network request."""
+        from renku.core.utils import requests
+
         record_id = ZenodoProvider.record_id(uri)
 
-        with retry() as session:
-            response = session.get(make_records_url(record_id), headers={"Accept": self._accept})
-            if response.status_code != 200:
-                raise LookupError("record not found. Status: {}".format(response.status_code))
-            return response
+        response = requests.get(make_records_url(record_id), headers={"Accept": self._accept})
+        if response.status_code != 200:
+            raise LookupError("record not found. Status: {}".format(response.status_code))
+        return response
 
     def find_record(self, uri, client=None, **kwargs):
         """Retrieves a record from Zenodo.
