@@ -22,15 +22,22 @@ from pathlib import Path
 from subprocess import run
 
 from renku.core import errors
-from renku.core.incubation.command import Command
+from renku.core.management.command_builder import inject
+from renku.core.management.command_builder.command import Command
+from renku.core.management.dataset.datasets_provenance import DatasetsProvenance
+from renku.core.management.interface.client_dispatcher import IClientDispatcher
 from renku.core.utils import communication
+from renku.core.utils.git import get_git_user
 
 
-def _remove(client, sources, edit_command):
+@inject.autoparams()
+def _remove(sources, edit_command, client_dispatcher: IClientDispatcher):
     """Remove files and check repository for potential problems."""
     from renku.core.management.git import _expand_directories
 
-    def fmt_path(path):
+    client = client_dispatcher.current_client
+
+    def get_relative_path(path):
         """Format path as relative to the client path."""
         abs_path = os.path.abspath(client.path / path)
         try:
@@ -39,7 +46,7 @@ def _remove(client, sources, edit_command):
             raise errors.ParameterError(f"File {abs_path} is not within the project.")
 
     files = {
-        fmt_path(source): fmt_path(file_or_dir)
+        get_relative_path(source): get_relative_path(file_or_dir)
         for file_or_dir in sources
         for source in _expand_directories((file_or_dir,))
     }
@@ -50,27 +57,31 @@ def _remove(client, sources, edit_command):
     try:
         for dataset in client.datasets.values():
             remove = []
-            for file_ in dataset.files:
-                key = file_.path
-                filepath = fmt_path(file_.path)
+            for file in dataset.files:
+                key = file.entity.path
+                filepath = get_relative_path(key)
                 if filepath in files:
                     remove.append(key)
 
             if remove:
+                dataset = dataset.copy()
                 for key in remove:
                     dataset.unlink_file(key)
                     client.remove_file(client.path / key)
 
-                dataset.to_yaml()
+                datasets_provenance = DatasetsProvenance()
+                datasets_provenance.add_or_update(dataset, creator=get_git_user(client.repository))
             communication.update_progress(progress_text, amount=1)
     finally:
         communication.finalize_progress(progress_text)
 
     # 2. Manage .gitattributes for external storage.
     if client.check_external_storage():
-        tracked = tuple(path for path, attr in client.find_attr(*files).items() if attr.get("filter") == "lfs")
+        tracked = tuple(
+            path for path, attr in client.repository.get_attributes(*files).items() if attr.get("filter") == "lfs"
+        )
         client.untrack_paths_from_storage(*tracked)
-        existing = client.find_attr(*tracked)
+        existing = client.repository.get_attributes(*tracked)
         if existing:
             communication.warn("There are custom .gitattributes.\n")
             if communication.confirm('Do you want to edit ".gitattributes" now?'):
@@ -85,4 +96,4 @@ def _remove(client, sources, edit_command):
 
 def remove_command():
     """Command to remove a file."""
-    return Command().command(_remove).require_clean().with_commit()
+    return Command().command(_remove).require_clean().with_database(write=True).with_commit()
