@@ -24,16 +24,13 @@ from pathlib import Path
 import attr
 import click
 import portalocker
-from pkg_resources import resource_filename
 
+from renku.core.management import RENKU_HOME
 from renku.core.models.enums import ConfigFilter
 
 APP_NAME = "Renku"
 """Application name for storing configuration."""
 
-RENKU_HOME = ".renku"
-"""Project directory name."""
-
 
 def _get_global_config_dir():
     """Return user's config directory."""
@@ -60,7 +57,7 @@ class ConfigManagerMixin:
         """Renku global (user's) config path."""
         config = Path(self.global_config_dir)
         if not config.exists():
-            config.mkdir()
+            config.mkdir(parents=True)
 
         return str(config / Path(self.CONFIG_NAME))
 
@@ -81,8 +78,16 @@ class ConfigManagerMixin:
 
     def load_config(self, config_filter=ConfigFilter.ALL):
         """Loads local, global or both configuration object."""
-        config = configparser.ConfigParser()
-        config_files = [resource_filename("renku", "data/defaults.ini")]
+        try:
+            import importlib_resources
+        except ImportError:
+            import importlib.resources as importlib_resources
+
+        # NOTE: Use RawConfigParser because ConfigParser does non-standard INI interpolation of some values
+        config = configparser.RawConfigParser()
+        ref = importlib_resources.files("renku.data") / "defaults.ini"
+        with importlib_resources.as_file(ref) as default_ini:
+            config_files = [default_ini]
 
         if config_filter == ConfigFilter.LOCAL_ONLY:
             config_files += [self.local_config_path]
@@ -120,11 +125,9 @@ class ConfigManagerMixin:
             with self.global_config_write_lock:
                 os.umask(0)
                 fd = os.open(filepath, os.O_CREAT | os.O_RDWR | os.O_TRUNC, 0o600)
-                with open(fd, "w+") as file:
-                    config.write(file)
+                self._write_config(fd, config)
         else:
-            with open(filepath, "w+") as file:
-                config.write(file)
+            self._write_config(filepath, config)
 
     def get_config(self, config_filter=ConfigFilter.ALL, as_string=True):
         """Read all configurations."""
@@ -189,6 +192,10 @@ class ConfigManagerMixin:
             self.store_config(config, global_only=global_only)
         return value
 
+    def _write_config(self, filepath, config):
+        with open(filepath, "w+") as file:
+            config.write(file)
+
     def _check_config_is_not_readonly(self, section, key):
         from renku.core import errors
 
