@@ -20,17 +20,23 @@
 import stat
 from pathlib import Path
 
-import pkg_resources
-from git.index.fun import hook_path as get_hook_path
+from renku.core.management.command_builder.command import inject
+from renku.core.management.interface.client_dispatcher import IClientDispatcher
+from renku.core.utils.git import get_hook_path
+
+try:
+    import importlib_resources
+except ImportError:
+    import importlib.resources as importlib_resources
 
 HOOKS = ("pre-commit",)
 
 
-def install(client, force):
+def install(force, repository):
     """Install Git hooks."""
     warning_messages = []
     for hook in HOOKS:
-        hook_path = Path(get_hook_path(hook, client.repo.git_dir))
+        hook_path = get_hook_path(name=hook, repository=repository)
         if hook_path.exists():
             if not force:
                 warning_messages.append("Hook already exists. Skipping {0}".format(str(hook_path)))
@@ -40,16 +46,19 @@ def install(client, force):
 
         # Make sure the hooks directory exists.
         hook_path.parent.mkdir(parents=True, exist_ok=True)
-
-        Path(hook_path).write_bytes(pkg_resources.resource_string("renku.data", "{hook}.sh".format(hook=hook)))
+        hook_data = importlib_resources.files("renku.data").joinpath(f"{hook}.sh").read_bytes()
+        Path(hook_path).write_bytes(hook_data)
         hook_path.chmod(hook_path.stat().st_mode | stat.S_IEXEC)
 
     return warning_messages
 
 
-def uninstall(client):
+@inject.autoparams()
+def uninstall(client_dispatcher: IClientDispatcher):
     """Uninstall Git hooks."""
+    client = client_dispatcher.current_client
+
     for hook in HOOKS:
-        hook_path = Path(get_hook_path(hook, client.repo.git_dir))
+        hook_path = get_hook_path(name=hook, repository=client.repository)
         if hook_path.exists():
             hook_path.unlink()
