@@ -19,27 +19,36 @@
 
 import glob
 import os
+import traceback
 import uuid
 from collections import defaultdict
 from functools import cmp_to_key
 from hashlib import sha1
 from pathlib import Path
+from urllib.parse import urlparse
 
-from cwlgen import CommandLineTool, parse_cwl
-from cwlgen.requirements import InitialWorkDirRequirement
-from git import NULL_TREE, Actor
+from cwl_utils.parser import load_document_by_uri
+from cwl_utils.parser.cwl_v1_0 import CommandLineTool, InitialWorkDirRequirement
 from werkzeug.utils import secure_filename
 
+from renku.core import errors
 from renku.core.management.migrations.models.v3 import Dataset
-from renku.core.models.entities import Collection, Entity
-from renku.core.models.provenance.activities import ProcessRun, WorkflowRun
-from renku.core.models.provenance.agents import Person, SoftwareAgent
-from renku.core.models.workflow.parameters import CommandArgument, CommandInput, CommandOutput, MappedIOStream
-from renku.core.models.workflow.run import Run
+from renku.core.management.migrations.models.v9 import (
+    Collection,
+    CommandArgument,
+    CommandInput,
+    CommandOutput,
+    Entity,
+    MappedIOStream,
+    Person,
+    ProcessRun,
+    Run,
+    SoftwareAgent,
+    WorkflowRun,
+)
+from renku.core.management.migrations.utils import OLD_DATASETS_PATH, OLD_WORKFLOW_PATH, MigrationType
+from renku.core.metadata.repository import Actor, Commit, git_unicode_unescape
 from renku.core.utils import communication
-from renku.core.utils.git import add_to_git
-from renku.core.utils.migrate import MigrationType
-from renku.core.utils.scm import git_unicode_unescape
 from renku.version import __version__, version_url
 
 default_missing_software_agent = SoftwareAgent(
@@ -47,14 +56,14 @@ default_missing_software_agent = SoftwareAgent(
 )
 
 
-def migrate(client):
+def migrate(migration_context):
     """Migration function."""
-    if MigrationType.WORKFLOWS not in client.migration_type:
+    if MigrationType.WORKFLOWS not in migration_context.options.type:
         return
-    _migrate_old_workflows(client)
+    _migrate_old_workflows(client=migration_context.client, strict=migration_context.options.strict)
 
 
-def _migrate_old_workflows(client):
+def _migrate_old_workflows(client, strict):
     """Migrates old cwl workflows to new jsonld format."""
 
     def sort_cwl_commits(e1, e2):
@@ -62,12 +71,12 @@ def _migrate_old_workflows(client):
         commit1 = e1[1]
         commit2 = e2[1]
 
-        return _compare_commits(client, commit1, commit2)
+        return commit1.compare_to(commit2)
 
     cache = RepositoryCache.from_client(client)
     client.cache = cache
 
-    wf_path = f"{client.workflow_path}/*.cwl"
+    wf_path = f"{client.renku_path}/workflow/*.cwl"
     for path in glob.glob(wf_path):
         if path not in cache.cwl_files_commits:
             raise ValueError(f"Couldn't find a previous commit for path `{path}`")
@@ -79,22 +88,25 @@ def _migrate_old_workflows(client):
         communication.echo(f"Processing commit {n}/{len(cwl_paths)}", end="\r")
 
         cwl_file, commit = element
-        if not Path(cwl_file).exists():
-            continue
-        path = _migrate_cwl(client, cwl_file, commit)
-        os.remove(cwl_file)
 
-        paths = [cwl_file, path]
-        if client.activity_index_path.exists():
-            paths.append(client.activity_index_path)
-        add_to_git(client.repo.git, *paths)
+        try:
+            if not Path(cwl_file).exists():
+                continue
 
-        if client.repo.is_dirty():
-            commit_msg = "renku migrate: " "committing migrated workflow"
+            path = _migrate_cwl(client, cwl_file, commit)
+            os.remove(cwl_file)
 
-            committer = Actor("renku {0}".format(__version__), version_url)
+            client.repository.add(cwl_file, path)
 
-            client.repo.index.commit(commit_msg, committer=committer, skip_hooks=True)
+            if client.repository.is_dirty():
+                commit_msg = "renku migrate: committing migrated workflow"
+                committer = Actor(name=f"renku {__version__}", email=version_url)
+                client.repository.commit(commit_msg + client.transaction_id, committer=committer, no_verify=True)
+        except Exception:
+            if strict:
+                raise
+            communication.echo("")
+            communication.warn(f"Cannot process commit '{commit.hexsha}' - Exception: {traceback.format_exc()}")
 
 
 def _migrate_cwl(client, path, commit):
@@ -138,10 +150,10 @@ def _migrate_single_step(client, cmd_line_tool, path, commit=None, parent_commit
         if name.endswith(")"):
             name = name[:-1]
 
-        matched_input = next(i for i in inputs if i.id == name)
+        matched_input = next(i for i in inputs if i.id.endswith(name))
         inputs.remove(matched_input)
 
-        path = client.workflow_path / Path(matched_input.default["path"])
+        path = client.renku_path / OLD_WORKFLOW_PATH / Path(matched_input.default["path"])
         stdin = path.resolve().relative_to(client.path)
         id_ = CommandInput.generate_id(base_id, "stdin")
 
@@ -163,7 +175,7 @@ def _migrate_single_step(client, cmd_line_tool, path, commit=None, parent_commit
             )
         )
 
-        matched_output = next(o for o in outputs if o.id == "output_stdout")
+        matched_output = next(o for o in outputs if o.id.endswith("output_stdout"))
 
         if matched_output:
             outputs.remove(matched_output)
@@ -178,7 +190,7 @@ def _migrate_single_step(client, cmd_line_tool, path, commit=None, parent_commit
             )
         )
 
-        matched_output = next(o for o in outputs if o.id == "output_stderr")
+        matched_output = next(o for o in outputs if o.id.endswith("output_stderr"))
 
         if matched_output:
             outputs.remove(matched_output)
@@ -204,14 +216,14 @@ def _migrate_single_step(client, cmd_line_tool, path, commit=None, parent_commit
             if name.endswith(")"):
                 name = name[:-1]
 
-            matched_input = next(i for i in all_inputs if i.id == name)
+            matched_input = next(i for i in all_inputs if i.id.endswith(name))
             try:
                 inputs.remove(matched_input)
             except ValueError:
                 pass
 
             if isinstance(matched_input.default, dict):
-                path = client.workflow_path / Path(matched_input.default["path"])
+                path = client.renku_path / OLD_WORKFLOW_PATH / Path(matched_input.default["path"])
             else:
                 path = Path(matched_input.default)
 
@@ -257,7 +269,7 @@ def _migrate_single_step(client, cmd_line_tool, path, commit=None, parent_commit
                 prefix += " "
 
         if isinstance(i.default, dict) and "class" in i.default and i.default["class"] in ["File", "Directory"]:
-            path = client.workflow_path / Path(i.default["path"])
+            path = client.renku_path / OLD_WORKFLOW_PATH / Path(i.default["path"])
             path = Path(os.path.realpath(path)).relative_to(client.path)
 
             run.inputs.append(
@@ -279,15 +291,15 @@ def _migrate_single_step(client, cmd_line_tool, path, commit=None, parent_commit
             )
 
     for a in cmd_line_tool.arguments:
-        id_ = CommandArgument.generate_id(base_id, a["position"])
-        run.arguments.append(CommandArgument(id=id_, position=a["position"], value=a["valueFrom"]))
+        id_ = CommandArgument.generate_id(base_id, a.position)
+        run.arguments.append(CommandArgument(id=id_, position=a.position, value=a.valueFrom))
 
     if not persist:
         return run, None
 
     step_name = "{0}_{1}.yaml".format(uuid.uuid4().hex, secure_filename("_".join(cmd_line_tool.baseCommand)))
 
-    absolute_path = client.workflow_path / step_name
+    absolute_path = client.renku_path / OLD_WORKFLOW_PATH / step_name
     path = absolute_path.relative_to(client.path)
 
     run.path = path
@@ -300,7 +312,6 @@ def _migrate_single_step(client, cmd_line_tool, path, commit=None, parent_commit
     ):
         process_run.association.agent = default_missing_software_agent
     process_run.to_yaml(path=absolute_path)
-    client.add_to_activity_index(process_run)
     return process_run, absolute_path
 
 
@@ -316,14 +327,16 @@ def _migrate_composite_step(client, workflow, path, commit=None):
 
     name = "{0}_migrated.yaml".format(uuid.uuid4().hex)
 
-    run.path = (client.workflow_path / name).relative_to(client.path)
+    wf_path = client.renku_path / OLD_WORKFLOW_PATH
+    run.path = (wf_path / name).relative_to(client.path)
 
     for step in workflow.steps:
         if isinstance(step.run, dict):
             continue
         else:
-            path = client.workflow_path / step.run
-            subrun = parse_cwl_cached(str(path))
+            uri = urlparse(step.run)
+            path = uri.path
+            subrun = parse_cwl_cached(path)
 
         subprocess, _ = _migrate_single_step(client, subrun, path, parent_commit=commit)
         run.add_subprocess(subprocess)
@@ -337,36 +350,34 @@ def _migrate_composite_step(client, workflow, path, commit=None):
         if isinstance(p.association.agent, Person) or not p.association.agent.label.startswith("renku "):
             p.association.agent = default_missing_software_agent
     wf.to_yaml(run.path)
-    client.add_to_activity_index(wf)
 
     return wf, run.path
 
 
 def _entity_from_path(client, path, commit):
     """Gets the entity associated with a path."""
-    client, commit, path = client.resolve_in_submodules(client.cache.find_previous_commit(path, revision=commit), path)
+    client, commit, path = client.get_in_submodules(client.cache.find_previous_commit(path, revision=commit), path)
 
     entity_cls = Entity
     if (client.path / path).is_dir():
         entity_cls = Collection
 
-    if str(path).startswith(os.path.join(client.renku_home, client.DATASETS)):
+    if str(path).startswith(os.path.join(client.renku_home, OLD_DATASETS_PATH)):
         return Dataset.from_yaml(path=client.path / path, client=client, commit=commit)
     else:
         return entity_cls(commit=commit, client=client, path=str(path))
 
 
-def _invalidations_from_commit(client, commit):
+def _invalidations_from_commit(client, commit: Commit):
     """Gets invalidated files from a commit."""
     results = []
     collections = dict()
-    for file_ in commit.diff(commit.parents or NULL_TREE):
-        # only process deleted files (note they appear as ADDED)
-        # in this backwards diff
-        if file_.change_type != "A":
+    for file in commit.get_changes():
+        # NOTE: only process deleted files
+        if not file.deleted:
             continue
-        path_ = Path(git_unicode_unescape(file_.a_path))
-        entity = _get_activity_entity(client, commit, path_, collections, deleted=True)
+        path = Path(file.a_path)
+        entity = _get_activity_entity(client, commit, path, collections, deleted=True)
 
         results.append(entity)
 
@@ -375,7 +386,7 @@ def _invalidations_from_commit(client, commit):
 
 def _get_activity_entity(client, commit, path, collections, deleted=False):
     """Gets the entity associated with this Activity and path."""
-    client, commit, path = client.resolve_in_submodules(commit, path)
+    client, commit, path = client.get_in_submodules(commit, path)
     output_path = client.path / path
     parents = list(output_path.relative_to(client.path).parents)
 
@@ -395,7 +406,7 @@ def _get_activity_entity(client, commit, path, collections, deleted=False):
     if (client.path / path).is_dir():
         entity_cls = Collection
 
-    if str(path).startswith(os.path.join(client.renku_home, client.DATASETS)) and not deleted:
+    if str(path).startswith(os.path.join(client.renku_home, OLD_DATASETS_PATH)) and not deleted:
         entity = Dataset.from_yaml(path=client.path / path, client=client, commit=commit)
     else:
         entity = entity_cls(commit=commit, client=client, path=str(path), parent=collection)
@@ -414,7 +425,7 @@ def parse_cwl_cached(path):
     if path in _cwl_cache:
         return _cwl_cache[path]
 
-    cwl = parse_cwl(path)
+    cwl = load_document_by_uri(path)
 
     _cwl_cache[path] = cwl
 
@@ -422,7 +433,7 @@ def parse_cwl_cached(path):
 
 
 class RepositoryCache:
-    """Cache for a git repo."""
+    """Cache for a git repository."""
 
     def __init__(self, client, cache, cwl_files_commits):
         self.client = client
@@ -431,33 +442,33 @@ class RepositoryCache:
 
     @classmethod
     def from_client(cls, client):
-        """Return a cached repo."""
+        """Return a cached repository."""
         cache = defaultdict(list)
         cwl_files_commits_map = {}
 
-        for n, commit in enumerate(client.repo.iter_commits(full_history=True), start=1):
+        for n, commit in enumerate(client.repository.iterate_commits(full_history=True), start=1):
             communication.echo(f"Caching commit {n}", end="\r")
 
             cwl_files = []
-            for file in commit.diff(commit.parents or NULL_TREE):
-                # Ignore deleted files (they appear as ADDED in this backwards diff)
-                if file.change_type == "A":
+            for file in commit.get_changes():
+                # Ignore deleted files
+                if file.deleted:
                     continue
 
-                path = git_unicode_unescape(file.a_path)
+                path = file.b_path
                 cache[path].append(commit)
 
-                if path.startswith(client.cwl_prefix) and path.endswith(".cwl"):
+                if path.startswith(f"{client.renku_home}/workflow/") and path.endswith(".cwl"):
                     cwl_files.append(os.path.realpath(client.path / path))
 
-            cls._update_cwl_files_and_commits(client, commit, cwl_files_commits_map, cwl_files)
+            cls._update_cwl_files_and_commits(commit, cwl_files_commits_map, cwl_files)
 
         communication.echo(40 * " ", end="\r")
 
         return RepositoryCache(client, cache, cwl_files_commits_map)
 
     @staticmethod
-    def _update_cwl_files_and_commits(client, commit, cwl_files_commits_map, cwl_files):
+    def _update_cwl_files_and_commits(commit, cwl_files_commits_map, cwl_files):
         if len(cwl_files) != 1:
             return
 
@@ -466,7 +477,7 @@ class RepositoryCache:
 
         if existing_commit is None:
             cwl_files_commits_map[path] = commit
-        elif _compare_commits(client, existing_commit, commit) < 0:  # existing commit is older
+        elif existing_commit.compare_to(commit) < 0:  # existing commit is older
             cwl_files_commits_map[path] = commit
 
     def find_previous_commit(self, path, revision="HEAD"):
@@ -474,8 +485,8 @@ class RepositoryCache:
 
         def find_from_client(path, revision):
             try:
-                return self.client.find_previous_commit(paths=path, revision=revision, full=True)
-            except KeyError:
+                return self.client.repository.get_previous_commit(path=path, revision=revision, full_history=True)
+            except errors.GitCommitNotFoundError:
                 communication.warn(f"Cannot find previous commit for {path} from {str(revision)}")
                 return revision
 
@@ -486,7 +497,7 @@ class RepositoryCache:
         path = str(path)
 
         if revision == "HEAD":
-            revision = self.client.head.commit
+            revision = self.client.repository.head.commit
 
         commits = self.cache.get(git_unicode_unescape(path))
         if not commits:
@@ -496,29 +507,8 @@ class RepositoryCache:
             return revision
 
         for commit in commits:
-            if _compare_commits(self.client, commit, revision) <= 0:
+            if commit.compare_to(revision) <= 0:
                 return commit
 
         # No commit was found
         return find_from_client(path, revision)
-
-
-def _compare_commits(client, commit1, commit2):
-    """Return -1 if commit1 is made before commit2."""
-    if client.repo.is_ancestor(commit1, commit2):
-        return -1
-    if client.repo.is_ancestor(commit2, commit1):
-        return 1
-
-    if commit1.committed_date < commit2.committed_date:
-        return -1
-    if commit1.committed_date > commit2.committed_date:
-        return 1
-
-    if commit1.authored_date < commit2.authored_date:
-        return -1
-    if commit1.authored_date > commit2.authored_date:
-        return 1
-
-    # NOTE: There is no ordering between the commits and there is no easy way for users to fix this
-    return 0
