@@ -21,23 +21,24 @@ import functools
 import itertools
 import os
 import re
+import shlex
 import tempfile
 from collections import defaultdict
 from pathlib import Path
 from shutil import move, which
 from subprocess import PIPE, STDOUT, check_output, run
+from typing import Optional
 
 import attr
 import pathspec
 from werkzeug.utils import cached_property
 
 from renku.core import errors
-from renku.core.models.provenance.activities import Collection
-from renku.core.models.provenance.datasets import DatasetProvenance
-from renku.core.models.provenance.provenance_graph import ProvenanceGraph
+from renku.core.models.entity import Entity
+from renku.core.models.provenance.activity import Collection
 from renku.core.utils import communication
 from renku.core.utils.file_size import parse_file_size
-from renku.core.utils.git import add_to_git, run_command
+from renku.core.utils.git import run_command
 
 from .git import _expand_directories
 from .repository import RepositoryApiMixin
@@ -100,7 +101,7 @@ class StorageApiMixin(RepositoryApiMixin):
     @cached_property
     def storage_installed_locally(self):
         """Verify that git-lfs is installed for the project."""
-        repo_config = self.repo.config_reader(config_level="repository")
+        repo_config = self.repository.get_configuration(scope="local")
         return repo_config.has_section('filter "lfs"')
 
     def check_external_storage(self):
@@ -111,10 +112,10 @@ class StorageApiMixin(RepositoryApiMixin):
         """
         storage_installed = self.storage_installed_locally and self.storage_installed
         if self.external_storage_requested and not storage_installed:
-            raise errors.ExternalStorageDisabled(self.repo)
+            raise errors.ExternalStorageDisabled(self.repository)
 
         if self.storage_installed_locally and not self.storage_installed:
-            raise errors.ExternalStorageNotInstalled(self.repo)
+            raise errors.ExternalStorageNotInstalled(self.repository)
 
         return storage_installed
 
@@ -172,7 +173,7 @@ class StorageApiMixin(RepositoryApiMixin):
 
         # Calculate which paths can be tracked in lfs
         track_paths = []
-        attrs = self.find_attr(*paths)
+        attrs = self.repository.get_attributes(*paths)
 
         for path in paths:
             path = Path(path)
@@ -181,23 +182,22 @@ class StorageApiMixin(RepositoryApiMixin):
             if path.is_symlink():
                 continue
 
-            if path.is_absolute():
-                path = Path(path).relative_to(self.path)
-
             # Do not add files with filter=lfs in .gitattributes
             if attrs.get(str(path), {}).get("filter") == "lfs" or not (self.path / path).exists():
                 continue
 
+            relative_path = Path(path).relative_to(self.path) if path.is_absolute() else path
+
             if (
                 path.is_dir()
-                and not self.renku_lfs_ignore.match_file(path)
-                and not any(self.renku_lfs_ignore.match_tree(path))
+                and not self.renku_lfs_ignore.match_file(relative_path)
+                and not any(self.renku_lfs_ignore.match_tree(str(relative_path)))
             ):
                 track_paths.append(str(path / "**"))
-            elif not self.renku_lfs_ignore.match_file(path):
+            elif not self.renku_lfs_ignore.match_file(str(relative_path)):
                 file_size = os.path.getsize(str(os.path.relpath(self.path / path, os.getcwd())))
                 if file_size >= self.minimum_lfs_file_size:
-                    track_paths.append(str(path))
+                    track_paths.append(str(relative_path))
 
         if track_paths:
             try:
@@ -254,9 +254,9 @@ class StorageApiMixin(RepositoryApiMixin):
         """List paths tracked in lfs for a client."""
         client = client or self
 
-        if len(client.repo.remotes) < 1 or not client.repo.active_branch.tracking_branch():
-            raise errors.ConfigurationError(
-                f"No git remote is configured for {client.path} branch {client.repo.active_branch.name}."
+        if len(client.repository.remotes) < 1 or not client.repository.active_branch.remote_branch:
+            raise errors.GitConfigurationError(
+                f"No git remote is configured for {client.path} branch {client.repository.active_branch.name}."
                 + "Cleaning the storage cache would lead to a loss of data as "
                 + "it is not on a server. Please see "
                 + "https://www.atlassian.com/git/tutorials/syncing for "
@@ -277,14 +277,14 @@ class StorageApiMixin(RepositoryApiMixin):
         client_dict = defaultdict(list)
 
         for path in _expand_directories(paths):
-            client, commit, path = self.resolve_in_submodules(self.repo.commit(), path)
+            client, commit, path = self.get_in_submodules(self.repository.head.commit, path)
             try:
                 absolute_path = Path(path).resolve()
                 relative_path = absolute_path.relative_to(client.path)
             except ValueError:  # An external file
                 absolute_path = Path(os.path.abspath(path))
                 relative_path = absolute_path.relative_to(client.path)
-            client_dict[client.path].append(str(relative_path))
+            client_dict[client.path].append(shlex.quote(str(relative_path)))
 
         for client_path, paths in client_dict.items():
             result = run_command(
@@ -298,20 +298,20 @@ class StorageApiMixin(RepositoryApiMixin):
             )
 
             if result.returncode != 0:
-                raise errors.GitLFSError(f"Error executing 'git lfs pull: \n {result.stdout}")
+                raise errors.GitLFSError(f"Cannot pull LFS objects from server:\n {result.stdout}")
 
     @check_external_storage_wrapper
     def clean_storage_cache(self, *paths):
         """Remove paths from lfs cache."""
         client_dict = defaultdict(list)
         clients = {}
-        tracked_paths = defaultdict(list)
-        unpushed_paths = defaultdict(list)
+        tracked_paths = {}
+        unpushed_paths = {}
         untracked_paths = []
         local_only_paths = []
 
         for path in _expand_directories(paths):
-            client, commit, path = self.resolve_in_submodules(self.repo.commit(), path)
+            client, _, path = self.get_in_submodules(self.repository.head.commit, path)
             try:
                 absolute_path = Path(path).resolve()
                 relative_path = absolute_path.relative_to(client.path)
@@ -361,7 +361,7 @@ class StorageApiMixin(RepositoryApiMixin):
                 move(tmp_path, path)
 
                 # get lfs sha hash
-                old_pointer = client.repo.git.show("HEAD:{}".format(path))
+                old_pointer = client.repository.get_raw_content(path=path, revision="HEAD")
                 old_pointer = old_pointer.splitlines()[1]
                 old_pointer = old_pointer.split(" ")[1].split(":")[1]
 
@@ -373,7 +373,7 @@ class StorageApiMixin(RepositoryApiMixin):
                 object_path.unlink()
 
             # add paths so they don't show as modified
-            add_to_git(client.repo.git, *paths)
+            client.repository.add(*paths)
 
         return untracked_paths, local_only_paths
 
@@ -393,7 +393,7 @@ class StorageApiMixin(RepositoryApiMixin):
         if not self.external_storage_requested:
             return
 
-        attrs = self.find_attr(*paths)
+        attrs = self.repository.get_attributes(*paths)
         track_paths = []
 
         for path in paths:
@@ -446,19 +446,30 @@ class StorageApiMixin(RepositoryApiMixin):
 
     def check_lfs_migrate_info(self, everything=False):
         """Return list of file groups in history should be in LFS."""
-        ref = ["--everything"] if everything else ["--include-ref", self.repo.active_branch.name]
+        ref = ["--everything"] if everything else ["--include-ref", self.repository.active_branch.name]
 
         includes, excludes, above = self.get_lfs_migrate_filters()
 
+        ignore_pointers = ["--pointers", "ignore"]
+
         command = self._CMD_STORAGE_MIGRATE_INFO + ref + above + includes + excludes
 
         try:
-            lfs_output = run(command, stdout=PIPE, stderr=STDOUT, cwd=self.path, universal_newlines=True)
+            lfs_output = run(
+                command + ignore_pointers, stdout=PIPE, stderr=STDOUT, cwd=self.path, universal_newlines=True
+            )
         except (KeyboardInterrupt, OSError) as e:
             raise errors.GitError(f"Couldn't run 'git lfs migrate info':\n{e}")
 
         if lfs_output.returncode != 0:
-            raise errors.GitLFSError(f"Error executing 'git lfs migrate info: \n {lfs_output.stdout}")
+            # NOTE: try running without --pointers (old versions of git lfs)
+            try:
+                lfs_output = run(command, stdout=PIPE, stderr=STDOUT, cwd=self.path, universal_newlines=True)
+            except (KeyboardInterrupt, OSError) as e:
+                raise errors.GitError(f"Couldn't run 'git lfs migrate info':\n{e}")
+
+            if lfs_output.returncode != 0:
+                raise errors.GitLFSError(f"Error executing 'git lfs migrate info: \n {lfs_output.stdout}")
 
         groups = []
         files_re = re.compile(r"(.*\s+[\d.]+\s+\S+).*")
@@ -511,12 +522,12 @@ class StorageApiMixin(RepositoryApiMixin):
         repo_root = Path(".")
 
         for old_commit_sha, new_commit_sha in commit_sha_mapping:
-            old_commit = self.repo.commit(old_commit_sha)
-            new_commit = self.repo.commit(new_commit_sha)
+            old_commit = self.repository.get_commit(old_commit_sha)
+            new_commit = self.repository.get_commit(new_commit_sha)
             processed = set()
 
-            for path in old_commit.stats.files.keys():
-                path_obj = Path(path)
+            for diff in old_commit.get_changes():
+                path_obj = Path(diff.b_path)
 
                 # NOTE: Get git object hash mapping for files and parent folders
                 while path_obj != repo_root:
@@ -532,45 +543,87 @@ class StorageApiMixin(RepositoryApiMixin):
                     processed.add(path_obj)
                     path_obj = path_obj.parent
 
-        def _map_checksum(entity, checksum_mapping):
+        def _map_checksum(entity, checksum_mapping) -> Optional[Entity]:
             """Update the checksum and id of an entity based on a mapping."""
             if entity.checksum not in checksum_mapping:
                 return
 
             new_checksum = checksum_mapping[entity.checksum]
 
+            if isinstance(entity, Collection) and entity.members:
+                members = []
+                for member in entity.members:
+                    new_member = _map_checksum(member, checksum_mapping)
+                    if new_member:
+                        member.append(new_member)
+                    else:
+                        members.append(member)
+                new_entity = Collection(checksum=new_checksum, path=entity.path, members=members)
+            else:
+                new_entity = Entity(checksum=new_checksum, path=entity.path)
+
+            return new_entity
+
+        def _map_checksum_old(entity, checksum_mapping):
+            """Update the checksum and id of an entity based on a mapping."""
+            # TODO: Remove this method once moved to Entity with 'id' field
+            from renku.core.models.provenance.activity import Collection
+
+            if entity.checksum not in checksum_mapping:
+                return
+
+            new_checksum = checksum_mapping[entity.checksum]
+
             entity._id = entity._id.replace(entity.checksum, new_checksum)
             entity.checksum = new_checksum
 
             if isinstance(entity, Collection) and entity.members:
                 for member in entity.members:
-                    _map_checksum(member, checksum_mapping)
+                    _map_checksum_old(member, checksum_mapping)
 
         # NOTE: Update workflow provenance
-        provenance_graph = ProvenanceGraph.from_json(self.provenance_graph_path)
+        # provenance_graph = ProvenanceGraph.from_database(database)
 
-        for activity in provenance_graph.activities:
-            if activity.generations:
-                for generation in activity.generations:
-                    entity = generation.entity
-                    _map_checksum(entity, sha_mapping)
+        # TODO: Update activities
+        # for activity in provenance_graph.activities:
+        #     # NOTE: This is a valid use-case since history will be re-written
+        #     activity.unfreeze()
+        #     if activity.generations:
+        #         generations = []
+        #         for generation in activity.generations:
+        #             new_entity = _map_checksum(generation.entity, sha_mapping)
+        #             if new_entity:
+        #                 new_generation = Generation(id=generation.id, entity=new_entity)
+        #                 generations.append(new_generation)
+        #             else:
+        #                 generations.append(generation)
+        #         activity.generations = generations
 
-            if activity.usages:
-                for usage in activity.usages:
-                    entity = usage.entity
-                    _map_checksum(entity, sha_mapping)
+        #     if activity.usages:
+        #         usages = []
+        #         for usage in activity.usages:
+        #             new_entity = _map_checksum(usage.entity, sha_mapping)
+        #             if new_entity:
+        #                 new_usage = Usage(id=usage.id, entity=new_entity)
+        #                 usages.append(new_usage)
+        #             else:
+        #                 usages.append(usage)
+        #         activity.usages = usages
 
-            if activity.invalidations:
-                for entity in activity.invalidations:
-                    _map_checksum(entity, sha_mapping)
+        #     if activity.invalidations:
+        #         invalidations = []
+        #         for entity in activity.invalidations:
+        #             new_entity = _map_checksum(entity, sha_mapping)
+        #             if new_entity:
+        #                 invalidations.append(new_entity)
+        #             else:
+        #                 invalidations.append(entity)
+        #         activity.invalidations = invalidations
 
-        provenance_graph.to_json()
+        #     activity.freeze()
 
         # NOTE: Update datasets provenance
-        datasets_provenance = DatasetProvenance.from_json(self.datasets_provenance_path)
-
-        for dataset in datasets_provenance.datasets:
-            for file_ in dataset.files:
-                _map_checksum(file_.entity, sha_mapping)
-
-        datasets_provenance.to_json()
+        # TODO: Fix dataset provenance
+        # for dataset in datasets_provenance.datasets:
+        #     for file_ in dataset.files:
+        #         _map_checksum_old(file_.entity, sha_mapping)
