@@ -18,12 +18,14 @@
 """Renku service migrate project controller."""
 from renku.core.utils.contexts import click_context
 from renku.service.cache.models.job import Job
+from renku.service.config import MESSAGE_PREFIX, PROJECT_CLONE_NO_DEPTH
 from renku.service.controllers.api.abstract import ServiceCtrl
 from renku.service.controllers.api.mixins import RenkuOpSyncMixin
 from renku.service.logger import worker_log
 from renku.service.serializers.cache import ProjectMigrateRequest, ProjectMigrateResponseRPC
 from renku.service.utils.callback import ServiceCallback
 from renku.service.views import result_response
+from renku.version import __version__
 
 
 def execute_migration(
@@ -70,10 +72,15 @@ class MigrateProjectCtrl(ServiceCtrl, RenkuOpSyncMixin):
         self.skip_template_update = self.ctx.get("skip_template_update", False)
         self.skip_docker_update = self.ctx.get("skip_docker_update", False)
         self.skip_migrations = self.ctx.get("skip_migrations", False)
-        self.commit_message = self.ctx.get("commit_message", None)
+        self.commit_message = f"{MESSAGE_PREFIX} migrate to latest version for renku {__version__}"
 
         super(MigrateProjectCtrl, self).__init__(
-            cache, user_data, request_data, migrate_project=migrate_project, skip_lock=skip_lock
+            cache,
+            user_data,
+            request_data,
+            migrate_project=migrate_project,
+            skip_lock=skip_lock,
+            clone_depth=PROJECT_CLONE_NO_DEPTH,
         )
 
     @property
@@ -92,23 +99,22 @@ class MigrateProjectCtrl(ServiceCtrl, RenkuOpSyncMixin):
             self.commit_message,
         )
 
+        return messages, was_migrated, template_migrated, docker_migrated
+
+    def to_response(self):
+        """Execute controller flow and serialize to service response."""
+        op_result, remote_branch = self.execute_and_sync()
+        if isinstance(op_result, Job):
+            return result_response(MigrateProjectCtrl.JOB_RESPONSE_SERIALIZER, op_result)
+
+        messages, was_migrated, template_migrated, docker_migrated = op_result
+
         response = {
             "messages": messages,
             "was_migrated": was_migrated,
             "template_migrated": template_migrated,
             "docker_migrated": docker_migrated,
+            "remote_branch": remote_branch,
         }
 
-        if was_migrated or template_migrated or docker_migrated:
-            response["remote_branch"] = self.sync()
-
-        return response
-
-    def to_response(self):
-        """Execute controller flow and serialize to service response."""
-        result = self.execute_op()
-
-        if isinstance(result, Job):
-            return result_response(MigrateProjectCtrl.JOB_RESPONSE_SERIALIZER, result)
-
-        return result_response(MigrateProjectCtrl.RESPONSE_SERIALIZER, result)
+        return result_response(MigrateProjectCtrl.RESPONSE_SERIALIZER, response)
