@@ -18,12 +18,13 @@
 """Renku service template create project controller."""
 import shutil
 
-import git
 from marshmallow import EXCLUDE
 
 from renku.core.commands.init import create_from_template_local_command, read_template_manifest
 from renku.core.errors import RenkuException
+from renku.core.metadata.repository import Repository
 from renku.core.utils.contexts import click_context
+from renku.service.config import MESSAGE_PREFIX
 from renku.service.controllers.api.abstract import ServiceCtrl
 from renku.service.controllers.api.mixins import RenkuOperationMixin
 from renku.service.controllers.utils.project_clone import user_project_clone
@@ -42,6 +43,7 @@ class TemplatesCreateProjectCtrl(ServiceCtrl, RenkuOperationMixin):
     def __init__(self, cache, user_data, request_data):
         """Construct a templates read manifest controller."""
         self.ctx = TemplatesCreateProjectCtrl.REQUEST_SERIALIZER.load({**user_data, **request_data}, unknown=EXCLUDE)
+        self.ctx["commit_message"] = f"{MESSAGE_PREFIX} init {self.ctx['project_name']}"
         super(TemplatesCreateProjectCtrl, self).__init__(cache, user_data, request_data)
 
         self.template = None
@@ -67,6 +69,7 @@ class TemplatesCreateProjectCtrl(ServiceCtrl, RenkuOperationMixin):
             "__repository__": self.ctx["project_repository"],
             "__sanitized_project_name__": self.ctx["project_name_stripped"],
             "__project_slug__": self.ctx["project_slug"],
+            "__project_description__": self.ctx["project_description"],
         }
         if is_release():
             metadata["__renku_version__"] = __version__
@@ -87,7 +90,9 @@ class TemplatesCreateProjectCtrl(ServiceCtrl, RenkuOperationMixin):
         new_project_data = {
             "clone_depth": self.ctx["depth"],
             "git_url": self.ctx["new_project_url"],
-            "name": self.ctx["project_name_stripped"],
+            "name": self.ctx["project_name"],
+            "slug": self.ctx["project_name_stripped"],
+            "description": self.ctx["project_description"],
             "fullname": self.ctx["fullname"],
             "email": self.ctx["email"],
             "owner": self.ctx["project_namespace"],
@@ -112,8 +117,8 @@ class TemplatesCreateProjectCtrl(ServiceCtrl, RenkuOperationMixin):
         if self.template is None:
             raise RenkuException("invalid identifier for target repository")
 
-        repo = git.Repo(str(project.abs_path))
-        self.template_version = repo.head.commit.hexsha
+        repository = Repository(project.abs_path)
+        self.template_version = repository.head.commit.hexsha
 
         # Verify missing parameters
         template_parameters = self.template.get("variables", {})
@@ -140,16 +145,19 @@ class TemplatesCreateProjectCtrl(ServiceCtrl, RenkuOperationMixin):
             create_from_template_local_command().build().execute(
                 source_path,
                 self.ctx["project_name"],
-                provided_parameters,
-                self.default_metadata,
-                self.template_version,
-                self.template.get("immutable_template_files", []),
-                self.template.get("allow_template_update", False),
-                self.git_user,
-                self.ctx["url"],
-                self.ctx["ref"],
-                "service",
-                self.ctx["initial_branch"],
+                metadata=provided_parameters,
+                default_metadata=self.default_metadata,
+                custom_metadata=self.ctx["project_custom_metadata"],
+                template_version=self.template_version,
+                immutable_template_files=self.template.get("immutable_template_files", []),
+                automated_template_update=self.template.get("allow_template_update", False),
+                user=self.git_user,
+                source=self.ctx["url"],
+                ref=self.ctx["ref"],
+                invoked_from="service",
+                initial_branch=self.ctx["initial_branch"],
+                commit_message=self.ctx["commit_message"],
+                description=self.ctx["project_description"],
             )
 
         self.new_project_push(new_project_path)
@@ -160,7 +168,8 @@ class TemplatesCreateProjectCtrl(ServiceCtrl, RenkuOperationMixin):
         return {
             "url": self.ctx["new_project_url"],
             "namespace": self.ctx["project_namespace"],
-            "name": self.ctx["project_name_stripped"],
+            "name": self.ctx["project_name"],
+            "slug": self.ctx["project_name_stripped"],
             "project_id": new_project.project_id,
         }
 
