@@ -41,6 +41,7 @@ from renku.service.views.config import config_blueprint
 from renku.service.views.datasets import dataset_blueprint
 from renku.service.views.graph import graph_blueprint
 from renku.service.views.jobs import jobs_blueprint
+from renku.service.views.project import project_blueprint
 from renku.service.views.templates import templates_blueprint
 from renku.service.views.version import version_blueprint
 
@@ -50,6 +51,7 @@ if os.getenv("SENTRY_DSN"):
     sentry_sdk.init(
         dsn=os.getenv("SENTRY_DSN"),
         environment=os.getenv("SENTRY_ENV"),
+        traces_sample_rate=float(os.getenv("SENTRY_SAMPLE_RATE", 0.2)),
         integrations=[FlaskIntegration(), RqIntegration(), RedisIntegration()],
     )
 
@@ -93,6 +95,7 @@ def build_routes(app):
     app.register_blueprint(dataset_blueprint)
     app.register_blueprint(graph_blueprint)
     app.register_blueprint(jobs_blueprint)
+    app.register_blueprint(project_blueprint)
     app.register_blueprint(templates_blueprint)
     app.register_blueprint(version_blueprint)
     app.register_blueprint(apispec_blueprint)
