@@ -18,21 +18,23 @@
 """Dataset jobs."""
 import urllib
 
-from git import GitCommandError, Repo
 from urllib3.exceptions import HTTPError
 
+from renku.core import errors
 from renku.core.commands.dataset import add_to_dataset, import_dataset
-from renku.core.commands.save import repo_sync
-from renku.core.errors import ParameterError, RenkuException
+from renku.core.metadata.repository import Repository
 from renku.core.models.git import GitURL
 from renku.core.utils.contexts import click_context
+from renku.core.utils.git import push_changes
 from renku.service.logger import worker_log
 from renku.service.utils.callback import ServiceCallback
 from renku.service.views.decorators import requires_cache
 
 
 @requires_cache
-def dataset_import(cache, user, user_job_id, project_id, dataset_uri, name=None, extract=False, timeout=None):
+def dataset_import(
+    cache, user, user_job_id, project_id, dataset_uri, name=None, extract=False, timeout=None, commit_message=None
+):
     """Job for dataset import."""
     user = cache.ensure_user(user)
     worker_log.debug(f"executing dataset import job for {user.user_id}:{user.fullname}")
@@ -49,18 +51,18 @@ def dataset_import(cache, user, user_job_id, project_id, dataset_uri, name=None,
 
             gitlab_token = user.token if _is_safe_to_pass_gitlab_token(project.git_url, dataset_uri) else None
 
-            command = import_dataset().with_commit_message(f"service: dataset import {dataset_uri}")
+            command = import_dataset().with_commit_message(commit_message)
             command.with_communicator(communicator).build().execute(
-                dataset_uri, name, extract, yes=True, gitlab_token=gitlab_token
+                uri=dataset_uri, name=name, extract=extract, yes=True, gitlab_token=gitlab_token
             )
 
             worker_log.debug("operation successful - syncing with remote")
-            _, remote_branch = repo_sync(Repo(project.abs_path), remote="origin")
+            remote_branch = push_changes(Repository(project.abs_path), remote="origin")
             user_job.update_extras("remote_branch", remote_branch)
 
             user_job.complete()
             worker_log.debug("job completed")
-    except (HTTPError, ParameterError, RenkuException, GitCommandError) as exp:
+    except (HTTPError, errors.ParameterError, errors.RenkuException, errors.GitCommandError) as exp:
         user_job.fail_job(str(exp))
 
         # Reraise exception, so we see trace in job metadata
@@ -99,12 +101,12 @@ def dataset_add_remote_file(cache, user, user_job_id, project_id, create_dataset
                 raise result.error
 
             worker_log.debug("operation successful - syncing with remote")
-            _, remote_branch = repo_sync(Repo(project.abs_path), remote="origin")
+            remote_branch = push_changes(Repository(project.abs_path), remote="origin")
             user_job.update_extras("remote_branch", remote_branch)
 
             user_job.complete()
             worker_log.debug("job completed")
-    except (HTTPError, BaseException, GitCommandError, RenkuException) as exp:
+    except (HTTPError, BaseException, errors.GitCommandError, errors.RenkuException) as exp:
         user_job.fail_job(str(exp))
 
         # Reraise exception, so we see trace in job metadata
