@@ -18,17 +18,23 @@
 """Renku service cache serializers."""
 import time
 import uuid
-from datetime import datetime
 from urllib.parse import urlparse
 
-import yagup
-from marshmallow import Schema, ValidationError, fields, post_load, pre_load, validates
+from marshmallow import Schema, ValidationError, fields, post_load, pre_load
 from werkzeug.utils import secure_filename
 
-from renku.core.errors import ConfigurationError
+from renku.core import errors
 from renku.core.models.git import GitURL
+from renku.core.utils.os import normalize_to_ascii
 from renku.service.config import PROJECT_CLONE_DEPTH_DEFAULT
-from renku.service.serializers.common import RenkuSyncSchema, RepositoryContext
+from renku.service.serializers.common import (
+    ArchiveSchema,
+    AsyncSchema,
+    FileDetailsSchema,
+    LocalRepositorySchema,
+    RemoteRepositorySchema,
+    RenkuSyncSchema,
+)
 from renku.service.serializers.rpc import JsonRPCResponse
 
 
@@ -50,35 +56,19 @@ def extract_file(request):
         return file
 
 
-class FileUploadRequest(Schema):
+class FileUploadRequest(ArchiveSchema):
     """Request schema for file upload."""
 
-    override_existing = fields.Boolean(missing=False)
-    unpack_archive = fields.Boolean(missing=False)
-
-
-class FileUploadContext(Schema):
-    """Context schema for file upload."""
-
-    created_at = fields.DateTime(missing=datetime.utcnow)
-    file_id = fields.String(missing=lambda: uuid.uuid4().hex)
-
-    content_type = fields.String(missing="unknown")
-    file_name = fields.String(required=True)
-
-    # measured in bytes (comes from stat() - st_size)
-    file_size = fields.Integer(required=True)
-
-    relative_path = fields.String(required=True)
-    is_archive = fields.Boolean(missing=False)
-    is_dir = fields.Boolean(required=True)
-    unpack_archive = fields.Boolean(missing=False)
+    override_existing = fields.Boolean(
+        missing=False,
+        description="Overried files. Useful when extracting from archives.",
+    )
 
 
 class FileUploadResponse(Schema):
     """Response schema for file upload."""
 
-    files = fields.List(fields.Nested(FileUploadContext), required=True)
+    files = fields.List(fields.Nested(FileDetailsSchema), required=True)
 
 
 class FileUploadResponseRPC(JsonRPCResponse):
@@ -90,7 +80,7 @@ class FileUploadResponseRPC(JsonRPCResponse):
 class FileListResponse(Schema):
     """Response schema for files listing."""
 
-    files = fields.List(fields.Nested(FileUploadContext), required=True)
+    files = fields.List(fields.Nested(FileDetailsSchema), required=True)
 
 
 class FileListResponseRPC(JsonRPCResponse):
@@ -99,42 +89,35 @@ class FileListResponseRPC(JsonRPCResponse):
     result = fields.Nested(FileListResponse)
 
 
-class RepositoryCloneRequest(Schema):
+class RepositoryCloneRequest(RemoteRepositorySchema):
     """Request schema for repository clone."""
 
-    git_url = fields.String(required=True)
-    depth = fields.Integer(missing=PROJECT_CLONE_DEPTH_DEFAULT)
-    ref = fields.String(missing=None)
+    depth = fields.Integer(description="Git fetch depth", missing=PROJECT_CLONE_DEPTH_DEFAULT)
+    ref = fields.String(description="Repository reference (branch, commit or tag)", missing=None)
 
 
-class RepositoryCloneContext(RepositoryCloneRequest):
-    """Context schema for a git repository clone."""
-
-    @validates("git_url")
-    def validate_git_url(self, value):
-        """Validates git url."""
-        try:
-            yagup.parse(value)
-        except yagup.exceptions.InvalidURL as e:
-            raise ValidationError("Invalid `git_url`") from e
-
-        return value
-
-
-class ProjectCloneContext(RepositoryCloneContext):
+class ProjectCloneContext(RepositoryCloneRequest):
     """Context schema for project clone."""
 
-    project_id = fields.String(missing=lambda: uuid.uuid4().hex)
-
     # measured in ms
     timestamp = fields.Integer(missing=time.time() * 1e3)
 
+    # user data
     name = fields.String()
+    slug = fields.String()
     fullname = fields.String()
     email = fields.String()
     owner = fields.String()
     token = fields.String()
 
+    @pre_load()
+    def set_missing_id(self, data, **kwargs):
+        """Set project_id when missing."""
+        if not data.get("project_id"):
+            data["project_id"] = lambda: uuid.uuid4().hex
+
+        return data
+
     @pre_load()
     def set_owner_name(self, data, **kwargs):
         """Set owner and name fields."""
@@ -142,7 +125,7 @@ class ProjectCloneContext(RepositoryCloneContext):
             git_url = GitURL.parse(data["git_url"])
         except UnicodeError as e:
             raise ValidationError("`git_url` contains unsupported characters") from e
-        except ConfigurationError as e:
+        except errors.InvalidGitURL as e:
             raise ValidationError("Invalid `git_url`") from e
 
         if git_url.owner is None:
@@ -152,6 +135,7 @@ class ProjectCloneContext(RepositoryCloneContext):
         if git_url.name is None:
             raise ValidationError("Invalid `git_url`")
         data["name"] = git_url.name
+        data["slug"] = normalize_to_ascii(data["name"])
 
         return data
 
@@ -210,7 +194,7 @@ class ProjectListResponseRPC(JsonRPCResponse):
     result = fields.Nested(ProjectListResponse)
 
 
-class ProjectMigrateRequest(RepositoryContext):
+class ProjectMigrateRequest(AsyncSchema, LocalRepositorySchema, RemoteRepositorySchema):
     """Request schema for project migrate."""
 
     force_template_update = fields.Boolean(default=False)
@@ -219,10 +203,14 @@ class ProjectMigrateRequest(RepositoryContext):
     skip_migrations = fields.Boolean(default=False)
 
     @pre_load()
-    def default_commit_message(self, data, **kwargs):
-        """Set default commit message."""
-        if not data.get("commit_message"):
-            data["commit_message"] = "service: renku migrate"
+    def handle_ref(self, data, **kwargs):
+        """Handle ref and branch."""
+
+        # Backward compatibility: branch and ref were both used. Let's keep branch as the exposed field
+        # even if interally it gets converted to "ref" later.
+        if data.get("ref"):
+            data["branch"] = data["ref"]
+            del data["ref"]
 
         return data
 
@@ -242,30 +230,82 @@ class ProjectMigrateResponseRPC(JsonRPCResponse):
     result = fields.Nested(ProjectMigrateResponse)
 
 
-class ProjectMigrationCheckRequest(Schema):
+class ProjectMigrationCheckRequest(LocalRepositorySchema, RemoteRepositorySchema):
     """Request schema for project migration check."""
 
-    project_id = fields.String()
 
-    git_url = fields.String()
-    branch = fields.String()
+class ProjectCompatibilityResponse(Schema):
+    """Response schema outlining service compatibility for migrations check."""
+
+    project_metadata_version = fields.String(description="Current version of the Renku metadata in the project.")
+    current_metadata_version = fields.String(description="Highest metadata version supported by this service.")
+    migration_required = fields.Boolean(
+        description="Whether or not a metadata migration is required to be compatible with this service."
+    )
+
+
+class DockerfileStatusResponse(Schema):
+    """Response schema outlining dockerfile status for migrations check."""
+
+    newer_renku_available = fields.Boolean(
+        description="Whether the version of Renku in this service is newer than the one in the Dockerfile."
+    )
+    automated_dockerfile_update = fields.Boolean(
+        description="Whether or not the Dockerfile supports automated Renku version updates."
+    )
+    latest_renku_version = fields.String(description="The current version of Renku available in this service.")
+    dockerfile_renku_version = fields.String(description="Version of Renku specified in the Dockerfile.")
+
+
+class TemplateStatusResponse(Schema):
+    """Response schema outlining template status for migrations check."""
+
+    automated_template_update = fields.Boolean(
+        description="Whether or not the project template explicitly supports automated updates."
+    )
+    newer_template_available = fields.Boolean(
+        description=(
+            "Whether or not the current version of the project template differs from the " "one used in the project."
+        )
+    )
+    template_source = fields.String(
+        description="Source of the template repository, either a Git URL or 'renku' if an embedded template was used."
+    )
+    template_ref = fields.String(
+        description="The branch/tag/commit from the template_source repository that was used to create this project."
+    )
+    template_id = fields.String(description="The id of the template in the template repository.")
+
+    project_template_version = fields.String(
+        allow_none=True, description="The version of the template last used in the user's project."
+    )
+    latest_template_version = fields.String(
+        allow_none=True, description="The current version of the template in the template repository."
+    )
 
 
 class ProjectMigrationCheckResponse(Schema):
     """Response schema for project migration check."""
 
-    migration_required = fields.Boolean()
-    template_update_possible = fields.Boolean()
-    automated_template_update = fields.Boolean()
-    current_template_version = fields.String(allow_none=True)
-    latest_template_version = fields.String(allow_none=True)
-    template_source = fields.String()
-    template_ref = fields.String()
-    template_id = fields.String()
-    docker_update_possible = fields.Boolean()
-    project_supported = fields.Boolean()
-    project_version = fields.String()
-    latest_version = fields.String()
+    project_supported = fields.Boolean(
+        description=(
+            "Determines whether this project is a Renku project that is supported by the version "
+            "running on this service (not made with a newer version)."
+        )
+    )
+    core_renku_version = fields.String(description="Version of Renku running in this service.")
+    project_renku_version = fields.String(description="Version of Renku last used to change the project.")
+
+    core_compatibility_status = fields.Nested(
+        ProjectCompatibilityResponse,
+        description="Fields detailing the compatibility of the project with this core service version.",
+    )
+    dockerfile_renku_status = fields.Nested(
+        DockerfileStatusResponse, description="Fields detailing the status of the Dockerfile in the project."
+    )
+    template_status = fields.Nested(
+        TemplateStatusResponse, description="Fields detailing the status of the project template used by this project."
+    )
 
 
 class ProjectMigrationCheckResponseRPC(JsonRPCResponse):
