@@ -25,20 +25,23 @@ from renku.service.controllers.cache_list_uploaded import ListUploadedFilesCtrl
 from renku.service.controllers.cache_migrate_project import MigrateProjectCtrl
 from renku.service.controllers.cache_migrations_check import MigrationsCheckCtrl
 from renku.service.controllers.cache_project_clone import ProjectCloneCtrl
-from renku.service.views.api_versions import V0_9, VersionedBlueprint
+from renku.service.gateways.gitlab_api_provider import GitlabAPIProvider
+from renku.service.views.api_versions import V0_9, V1_0, VersionedBlueprint
 from renku.service.views.decorators import (
     accepts_json,
     handle_common_except,
     handle_migration_except,
+    optional_identity,
     requires_cache,
     requires_identity,
 )
+from renku.service.views.v0_9.cache import add_v0_9_specific_endpoints
 
 CACHE_BLUEPRINT_TAG = "cache"
 cache_blueprint = VersionedBlueprint("cache", __name__, url_prefix=SERVICE_PREFIX)
 
 
-@cache_blueprint.route("/cache.files_list", methods=["GET"], provide_automatic_options=False, versions=[V0_9])
+@cache_blueprint.route("/cache.files_list", methods=["GET"], provide_automatic_options=False, versions=[V0_9, V1_0])
 @handle_common_except
 @requires_cache
 @requires_identity
@@ -61,7 +64,7 @@ def list_uploaded_files_view(user_data, cache):
     return ListUploadedFilesCtrl(cache, user_data).to_response()
 
 
-@cache_blueprint.route("/cache.files_upload", methods=["POST"], provide_automatic_options=False, versions=[V0_9])
+@cache_blueprint.route("/cache.files_upload", methods=["POST"], provide_automatic_options=False, versions=[V0_9, V1_0])
 @handle_common_except
 @requires_cache
 @requires_identity
@@ -96,7 +99,7 @@ def upload_file_view(user_data, cache):
     return UploadFilesCtrl(cache, user_data, request).to_response()
 
 
-@cache_blueprint.route("/cache.project_clone", methods=["POST"], provide_automatic_options=False, versions=[V0_9])
+@cache_blueprint.route("/cache.project_clone", methods=["POST"], provide_automatic_options=False, versions=[V0_9, V1_0])
 @handle_common_except
 @accepts_json
 @requires_cache
@@ -125,7 +128,7 @@ def project_clone_view(user_data, cache):
     return ProjectCloneCtrl(cache, user_data, dict(request.json)).to_response()
 
 
-@cache_blueprint.route("/cache.project_list", methods=["GET"], provide_automatic_options=False, versions=[V0_9])
+@cache_blueprint.route("/cache.project_list", methods=["GET"], provide_automatic_options=False, versions=[V0_9, V1_0])
 @handle_common_except
 @requires_cache
 @requires_identity
@@ -148,7 +151,7 @@ def list_projects_view(user_data, cache):
     return ListProjectsCtrl(cache, user_data).to_response()
 
 
-@cache_blueprint.route("/cache.migrate", methods=["POST"], provide_automatic_options=False, versions=[V0_9])
+@cache_blueprint.route("/cache.migrate", methods=["POST"], provide_automatic_options=False, versions=[V0_9, V1_0])
 @handle_common_except
 @handle_migration_except
 @accepts_json
@@ -177,11 +180,10 @@ def migrate_project_view(user_data, cache):
     return MigrateProjectCtrl(cache, user_data, dict(request.json)).to_response()
 
 
-@cache_blueprint.route("/cache.migrations_check", methods=["GET"], provide_automatic_options=False, versions=[V0_9])
+@cache_blueprint.route("/cache.migrations_check", methods=["GET"], provide_automatic_options=False, versions=[V1_0])
 @handle_common_except
 @requires_cache
-@requires_identity
-@accepts_json
+@optional_identity
 def migration_check_project_view(user_data, cache):
     """
     Retrieve migration information for a project.
@@ -201,4 +203,7 @@ def migration_check_project_view(user_data, cache):
       tags:
         - cache
     """
-    return MigrationsCheckCtrl(cache, user_data, dict(request.args)).to_response()
+    return MigrationsCheckCtrl(cache, user_data, dict(request.args), GitlabAPIProvider()).to_response()
+
+
+cache_blueprint = add_v0_9_specific_endpoints(cache_blueprint)
