@@ -28,7 +28,12 @@ from renku.service.jobs.queues import QUEUES, WorkerQueues
 from renku.service.logger import DEPLOYMENT_LOG_LEVEL, worker_log
 
 if os.getenv("SENTRY_DSN"):
-    sentry_sdk.init(dsn=os.getenv("SENTRY_DSN"), environment=os.getenv("SENTRY_ENV"), integrations=[RqIntegration()])
+    sentry_sdk.init(
+        dsn=os.getenv("SENTRY_DSN"),
+        environment=os.getenv("SENTRY_ENV"),
+        traces_sample_rate=float(os.getenv("SENTRY_SAMPLE_RATE", 0.2)),
+        integrations=[RqIntegration()],
+    )
 
 
 @contextmanager
@@ -74,7 +79,7 @@ if __name__ == "__main__":
 
     if not queues:
         raise ConfigurationError(
-            "Worker queues not specified. " "Please, set RENKU_SVC_WORKER_QUEUES environment variable."
+            "Worker queues not specified. Please, set RENKU_SVC_WORKER_QUEUES environment variable."
         )
 
     start_worker([queue_name.strip() for queue_name in queues.strip().split(",")])
